@@ -9,22 +9,29 @@
 
 import logging
 import pytest
+import os
+from lib389 import DirSrv, pid_from_file
+from lib389.dseldif import DSEldif
 from lib389.tasks import *
 from lib389.topologies import topology_m2, topology_st as topo
 from lib389.utils import *
 from lib389._constants import DN_CONFIG, DEFAULT_SUFFIX, DEFAULT_BENAME
+from lib389._mapped_object import DSLdapObjects
+from lib389.cli_base import FakeArgs
+from lib389.cli_conf.backend import db_config_set
 from lib389.idm.user import UserAccounts, TEST_USER_PROPERTIES
 from lib389.idm.group import Groups
-from lib389.backend import *
-from lib389.config import LDBMConfig, BDB_LDBMConfig
+from lib389.instance.setup import SetupDs
+from lib389.config import LDBMConfig, BDB_LDBMConfig, Config
 from lib389.cos import CosPointerDefinitions, CosTemplates
-from lib389.backend import Backends
-from lib389.monitor import MonitorLDBM
+from lib389.backend import Backends, DatabaseConfig
+from lib389.monitor import MonitorLDBM, Monitor
 from lib389.plugins import ReferentialIntegrityPlugin
 
 pytestmark = pytest.mark.tier0
 
 USER_DN = 'uid=test_user,%s' % DEFAULT_SUFFIX
+PSTACK_CMD = '/usr/bin/pstack'
 
 logging.getLogger(__name__).setLevel(logging.INFO)
 log = logging.getLogger(__name__)
@@ -41,8 +48,6 @@ def big_file():
     return TEMP_BIG_FILE
 
 
-@pytest.mark.bz1897248
-@pytest.mark.ds4315
 @pytest.mark.skipif(ds_is_older('1.4.3.16'), reason="This config setting exists in 1.4.3.16 and higher")
 def test_nagle_default_value(topo):
     """Test that nsslapd-nagle attribute is off by default
@@ -109,7 +114,10 @@ def test_maxbersize_repl(topology_m2, big_file):
 
     log.info('Check if a big value was successfully replicated to supplier2')
     photo_m2 = user_m2.get_attr_vals('jpegphoto')
-
+    nbtries = 0;
+    while photo_m2 != photo_m1 and nbtries < 10:
+        nbtries = nbtries + 1
+        photo_m2 = user_m2.get_attr_vals('jpegphoto')
     assert photo_m2 == photo_m1
 
 def test_config_listen_backport_size(topology_m2):
@@ -143,6 +151,7 @@ def test_config_listen_backport_size(topology_m2):
     topology_m2.ms["supplier1"].config.replace('nsslapd-listen-backlog-size', default_val)
 
 
+@pytest.mark.skipif(get_default_db_lib() == "mdb", reason="Not supported over mdb")
 def test_config_deadlock_policy(topology_m2):
     """Check that nsslapd-db-deadlock-policy acted as expected
 
@@ -194,8 +203,6 @@ def test_config_deadlock_policy(topology_m2):
     ldbmconfig.replace('nsslapd-db-deadlock-policy', deadlock_policy)
 
 
-@pytest.mark.bz766322
-@pytest.mark.ds26
 def test_defaultnamingcontext(topo):
     """Tests configuration attribute defaultNamingContext in the rootdse
 
@@ -280,27 +287,6 @@ def test_defaultnamingcontext(topo):
     b2.delete()
 
 
-@pytest.mark.xfail(reason="This may fail due to bug 1610234")
-def test_defaultnamingcontext_1(topo):
-    """This test case should be part of function test_defaultnamingcontext
-       Please move it back after we have a fix for bug 1610234
-    """
-    log.info("Remove the original suffix which is currently nsslapd-defaultnamingcontext"
-             "and check nsslapd-defaultnamingcontext become empty.")
-
-    """ Please remove these declarations after moving the test
-        to function test_defaultnamingcontext
-    """
-    backends = Backends(topo.standalone)
-    test_db2 = 'test2_db'
-    test_suffix2 = 'dc=test2,dc=com'
-    b2 = backends.create(properties={'cn': test_db2,
-                                     'nsslapd-suffix': test_suffix2})
-    b2.delete()
-    assert topo.standalone.config.get_attr_val_utf8('nsslapd-defaultnamingcontext') == ' '
-
-
-@pytest.mark.bz602456
 def test_allow_add_delete_config_attributes(topo):
     """Tests configuration attributes are allowed to add and delete
 
@@ -342,12 +328,11 @@ def test_allow_add_delete_config_attributes(topo):
     assert not topo.standalone.config.present('invalid-attribute', 'invalid-value')
 
 
-@pytest.mark.bz918705
-@pytest.mark.ds511
 def test_ignore_virtual_attrs(topo):
     """Test nsslapd-ignore-virtual-attrs configuration attribute
 
     :id: 9915d71b-2c71-4ac0-91d7-92655d53541b
+    :customerscenario: True
     :setup: Standalone instance
     :steps:
          1. Check the attribute nsslapd-ignore-virtual-attrs is present in cn=config
@@ -421,6 +406,7 @@ def test_ignore_virtual_attrs_after_restart(topo):
        its value on restart
 
     :id: ac368649-4fda-473c-9ef8-e0c728b162af
+    :customerscenario: True
     :setup: Standalone instance
     :steps:
          1. Check the attribute nsslapd-ignore-virtual-attrs is present in cn=config
@@ -450,10 +436,8 @@ def test_ignore_virtual_attrs_after_restart(topo):
     log.info("Check the default value of attribute nsslapd-ignore-virtual-attrs should be OFF")
     assert topo.standalone.config.present('nsslapd-ignore-virtual-attrs', 'off')
 
-@pytest.mark.bz918694
-@pytest.mark.ds408
 def test_ndn_cache_enabled(topo):
-    """Test nsslapd-ignore-virtual-attrs configuration attribute
+    """Check the behavior of the Normalized DN cache when enabled/disabled
 
     :id: 2caa3ec0-cd05-458e-9e21-3b73cf4697ff
     :setup: Standalone instance
@@ -515,7 +499,7 @@ def test_ndn_cache_enabled(topo):
 
 
 def test_require_index(topo):
-    """Test nsslapd-ignore-virtual-attrs configuration attribute
+    """Validate that unindexed searches are rejected
 
     :id: fb6e31f2-acc2-4e75-a195-5c356faeb803
     :setup: Standalone instance
@@ -549,7 +533,7 @@ def test_require_index(topo):
 
 @pytest.mark.skipif(ds_is_older('1.4.2'), reason="The config setting only exists in 1.4.2 and higher")
 def test_require_internal_index(topo):
-    """Test nsslapd-ignore-virtual-attrs configuration attribute
+    """Ensure internal operations require indexed attributes
 
     :id: 22b94f30-59e3-4f27-89a1-c4f4be036f7f
     :setup: Standalone instance
@@ -557,8 +541,7 @@ def test_require_internal_index(topo):
         1. Set "nsslapd-require-internalop-index" to "on"
         2. Enable RI plugin, and configure it to use an attribute that is not indexed
         3. Create a user and add it a group
-        4. Deleting user should be rejected as the RI plugin issues an
-        unindexed internal search
+        4. Deleting user should be rejected as the RI plugin issues an unindexed internal search
     :expectedresults:
         1. Success
         2. Success
@@ -605,6 +588,231 @@ def test_require_internal_index(topo):
         user.delete()
 
 
+def get_pstack(pid):
+    """Get a pstack of the pid."""
+    res = subprocess.run((PSTACK_CMD, str(pid)), stdout=subprocess.PIPE,
+                         stderr=subprocess.STDOUT, encoding='utf-8')
+    return str(res.stdout)
+
+def check_number_of_threads(cfgnbthreads, monitor, pid):
+    monresults = monitor.get_threads()
+    # Add waitingthreads and busythreads
+    waiting = int(monresults[3][0])
+    busy = int(monresults[4][0])
+    log.info('Number of threads: configured={cfgnbthreads} waiting={waiting} busy={busy}')
+
+    monnbthreads = int(monresults[3][0]) + int(monresults[4][0]);
+    assert monnbthreads == cfgnbthreads
+    if os.path.isfile(PSTACK_CMD):
+        pstackresult = get_pstack(pid)
+        assert pstackresult.count('connection_threadmain') == cfgnbthreads
+    else:
+        log.info('pstack is not installed ==> skipping pstack test.')
+
+def test_changing_threadnumber(topo):
+    """Validate thread number changes in the server configuration
+
+    :id: 11bcf426-061c-11ee-8c22-482ae39447e5
+    :setup: Standalone instance
+    :steps:
+        1. Check that feature is supported
+        2  Get nsslapd-threadnumber original value
+        3. Change nsslapd-threadnumber to 40
+        4. Check that monitoring and pstack shows the same number than configured number of threads
+        5. Create a user and add it a group
+        6. Change nsslapd-threadnumber to 10
+        7. Check that monitoring and pstack shows the same number than configured number of threads
+        8. Set back the number of threads to the original value
+        9. Check that monitoring and pstack shows the same number than configured number of threads
+    :expectedresults:
+        1. Skip the test if monitoring result does not have the new attributes.
+        2. Success
+        3. Success
+        4. Success
+        5. Success
+        6. Success
+        7. Success
+        8. Success
+        9. Success
+    """
+    inst = topo.standalone
+    pid = pid_from_file(inst.pid_file())
+    assert pid != 0 and pid != None
+
+    config = Config(inst)
+    cfgattr = 'nsslapd-threadnumber'
+    cfgnbthreads = config.get_attr_vals_utf8(cfgattr)[0]
+
+    monitor = Monitor(inst)
+    monresults = monitor.get_threads()
+    if len(monresults) < 5:
+        pytest.skip("This version does not support dynamic change of nsslapd-threadnumber without restart.")
+
+    config.replace(cfgattr, '40');
+    time.sleep(3)
+    check_number_of_threads(40, monitor, pid)
+
+    config.replace(cfgattr, '10');
+    # No need to wait here (threads are closed before config change result is returned)
+    check_number_of_threads(10, monitor, pid)
+
+    config.replace(cfgattr, cfgnbthreads);
+    time.sleep(3)
+    check_number_of_threads(int(cfgnbthreads), monitor, pid)
+
+
+@pytest.fixture(scope="module")
+def create_lmdb_instance(request):
+    verbose = log.level > logging.DEBUG
+    instname = 'i_lmdb'
+    assert SetupDs(verbose=True, log=log).create_from_dict( {
+        'general' : {},
+        'slapd' : {
+            'instance_name': instname,
+            'db_lib': 'mdb',
+            'mdb_max_size': '0.5 Gb',
+        },
+        'backend-userroot': {
+            'sample_entries': 'yes',
+            'suffix': DEFAULT_SUFFIX,
+        },
+    } )
+    inst = DirSrv(verbose=verbose, external_log=log)
+    inst.local_simple_allocate(instname, binddn=DN_DM, password=PW_DM)
+    inst.setup_ldapi()
+
+    def fin():
+        inst.delete()
+
+    request.addfinalizer(fin)
+    inst.open()
+    return inst
+
+
+def set_and_check(inst, db_config, dsconf_attr, ldap_attr, val):
+    val = str(val)
+    args = FakeArgs()
+    setattr(args, dsconf_attr, val)
+    db_config_set(inst, db_config.dn, log, args)
+    cfg_vals = db_config.get()
+    assert ldap_attr in cfg_vals
+    assert cfg_vals[ldap_attr][0] == val
+
+
+def test_lmdb_config(create_lmdb_instance):
+    """Verify LMDB configuration settings in custom instance
+
+    :id: bca28086-61cf-11ee-a064-482ae39447e5
+    :setup: Custom instance named 'i_lmdb' having db_lib=mdb and lmdb_size=0.5
+    :steps:
+        1. Get dscreate create-template output
+        2. Check that 'db_lib' is in output
+        3. Extract 'mdb_max_size' from output and verify it's correct
+        4. Get the database config
+        5. Check that nsslapd-backend-implement is mdb
+        6. Check that nsslapd-mdb-max-size is 536870912 (i.e 0.5Gb)
+        7. Set a value for nsslapd-mdb-max-size and test the value is properly set
+        8. Set a value for nsslapd-mdb-max-readers and test the value is properly set
+        9. Set a value for nsslapd-mdb-max-dbs and test the value is properly set
+    :expectedresults:
+        1. Success
+        2. Success
+        3. Success
+        4. Success
+        5. Success
+        6. Success
+        7. Success
+        8. Success
+        9. Success
+    """
+
+    res = subprocess.run(('dscreate', 'create-template'), stdout=subprocess.PIPE,
+                         stderr=subprocess.STDOUT, encoding='utf-8')
+    inst = create_lmdb_instance
+    assert 'db_lib' in res.stdout
+
+    # Extract the mdb_max_size value from the output
+    mdb_max_size_line = next(line for line in res.stdout.splitlines() if 'mdb_max_size =' in line)
+    mdb_max_size_value = mdb_max_size_line.split('=')[1].strip()
+
+    # Parse the mdb_max_size value to int and back
+    parsed_size = parse_size(mdb_max_size_value)
+    formatted_size = format_size(parsed_size)
+
+    # Assert the original value and the processed value are the same
+    assert mdb_max_size_value == formatted_size, f"Initial mdb_max_size value {mdb_max_size_value} does not match the formatted size {formatted_size}"
+
+    db_config = DatabaseConfig(inst)
+    cfg_vals = db_config.get()
+    assert 'nsslapd-backend-implement' in cfg_vals
+    assert cfg_vals['nsslapd-backend-implement'][0] == 'mdb'
+    assert 'nsslapd-mdb-max-size' in cfg_vals
+    assert cfg_vals['nsslapd-mdb-max-size'][0] == '536870912'
+    set_and_check(inst, db_config, 'mdb_max_size', 'nsslapd-mdb-max-size', parse_size('2G'))
+    set_and_check(inst, db_config, 'mdb_max_readers', 'nsslapd-mdb-max-readers', 200)
+    set_and_check(inst, db_config, 'mdb_max_dbs', 'nsslapd-mdb-max-dbs', 200)
+
+
+def test_numlisteners_limit(topo):
+    """Test higher limit of nsslapd-numlisteners than 4
+    DS allows a higher value of nsslapd-numlisteners than it's limit of 4
+
+    :id: 96869ea9-c7b4-4a4f-85f9-ea1d3f4a63aa
+    :setup: Standalone Instance
+    :steps:
+        1. Check default nsslapd-numlisteners value is 1
+        2. Set nsslapd-numlisteners value to 4
+        3. Check nsslapd-numlisteners value is set to 4
+        4. Check dse.ldif value of nsslapd-numlisteners is set to 4
+        5. systemctl restart dirsrv@localhost
+        6. Check nsslapd-numlisteners value is 4, after server restart
+        7. Set nsslapd-numlisteners value to greater than the max value (4)
+        8. Check if nsslapd-numlisteners value is still 4
+        9. Check dse.ldif value of nsslapd-numlisteners is set to 4
+    :expectedresults:
+        1. nsslapd-numlisteners config value should show 1 by default
+        2. nsslapd-numlisteners value should be successfully set to 4
+        3. nsslapd-numlisteners is indeed set to 4
+        4. nsslapd-numlisteners value in localhost dse.ldif file is set to 4
+        5. restart DS instance is successful
+        6. nsslapd-numlisteners value is still 4 after server restart
+        7. Invalid value is rejected
+        8. nsslapd-numlisteners value is still 4
+        9. nsslapd-numlisteners value in localhost dse.ldif file is set to 4
+    """
+    # Check default value for nsslapd-numlisteners is 1
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '1'
+
+    # Set nsslapd-numlisteners value to 4
+    topo.standalone.config.set('nsslapd-numlisteners', '4')
+
+    # Check nsslapd-numlisteners value is set to 4
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '4'
+
+    # Check instance dse.ldif value is set to 4
+    inst = topo.standalone
+    dse_ldif = DSEldif(inst)
+    numlisteners = dse_ldif.get(DN_CONFIG, 'nsslapd-numlisteners')
+    assert numlisteners[0] == '4'
+
+    # Restart instance
+    topo.standalone.restart()
+
+    # Check nsslapd-numlisteners value is set to 4
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '4'
+
+    # Check if nsslapd-numlisteners value is not set more than 4
+    with pytest.raises(ldap.UNWILLING_TO_PERFORM):
+        assert(not topo.standalone.config.replace('nsslapd-numlisteners', '5'))
+
+    # Check nsslapd-numlisteners value is set to 4
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '4'
+
+    # Check the value of nsslapd-numlisteners in dse.ldif is set to 4
+    numlisteners = dse_ldif.get(DN_CONFIG, 'nsslapd-numlisteners')
+    assert numlisteners[0] == '4'
+
+
 if __name__ == '__main__':
     # Run isolated
     # -s for DEBUG mode
