@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2020 Red Hat, Inc.
+# Copyright (C) 2023 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -15,6 +15,8 @@ import pytest
 import time
 import glob
 import logging
+import subprocess
+from datetime import datetime
 from lib389.topologies import topology_st as topo
 from lib389._constants import DEFAULT_SUFFIX, TaskWarning
 from lib389.dbgen import dbgen_users
@@ -23,9 +25,12 @@ from lib389.index import Indexes
 from lib389.monitor import Monitor
 from lib389.backend import Backends
 from lib389.config import LDBMConfig
-from lib389.utils import ds_is_newer
+from lib389.config import LMDB_LDBMConfig
+from lib389.utils import ds_is_newer, get_default_db_lib
 from lib389.idm.user import UserAccount
 from lib389.idm.account import Accounts
+from lib389.cli_ctl.dbtasks import dbtasks_ldif2db
+from lib389.cli_base import FakeArgs
 
 pytestmark = pytest.mark.tier1
 
@@ -36,6 +41,21 @@ else:
     logging.getLogger(__name__).setLevel(logging.INFO)
 log = logging.getLogger(__name__)
 
+bdb_values = {
+  'wait30': 30
+}
+
+# Note: I still sometime get failure with a 60s timeout so lets use 90s
+mdb_values = {
+  'wait30': 90
+}
+
+if get_default_db_lib() == 'bdb':
+    values = bdb_values
+else:
+    values = mdb_values
+
+
 def _generate_ldif(topo, no_no):
     """
     Will generate the ldifs
@@ -64,22 +84,26 @@ def _search_for_user(topo, no_n0):
     assert len(accounts.filter('(uid=*)')) == no_n0
 
 
+def _import_clean_topo(topo):
+    """
+    Cleanup after import
+    """
+    accounts = Accounts(topo.standalone, DEFAULT_SUFFIX)
+    for i in accounts.filter('(uid=*)'):
+        UserAccount(topo.standalone, i.dn).delete()
+
+    ldif_dir = topo.standalone.get_ldif_dir()
+    import_ldif = ldif_dir + '/basic_import.ldif'
+    if os.path.exists(import_ldif):
+        os.remove(import_ldif)
+    syntax_err_ldif = ldif_dir + '/syntax_err.dif'
+    if os.path.exists(syntax_err_ldif):
+        os.remove(syntax_err_ldif)
+
+
 @pytest.fixture(scope="function")
 def _import_clean(request, topo):
-    def finofaci():
-        accounts = Accounts(topo.standalone, DEFAULT_SUFFIX)
-        for i in accounts.filter('(uid=*)'):
-            UserAccount(topo.standalone, i.dn).delete()
-
-        ldif_dir = topo.standalone.get_ldif_dir()
-        import_ldif = ldif_dir + '/basic_import.ldif'
-        if os.path.exists(import_ldif):
-            os.remove(import_ldif)
-        syntax_err_ldif = ldif_dir + '/syntax_err.dif'
-        if os.path.exists(syntax_err_ldif):
-            os.remove(syntax_err_ldif)
-
-    request.addfinalizer(finofaci)
+    request.addfinalizer(lambda: _import_clean_topo(topo))
 
 
 def _import_offline(topo, no_no):
@@ -95,6 +119,7 @@ def _import_offline(topo, no_no):
     topo.standalone.stop()
     t1 = time.time()
     if not topo.standalone.ldif2db('userRoot', None, None, None, import_ldif):
+        topo.standalone.start()
         assert False
     total_time = time.time() - t1
     topo.standalone.start()
@@ -115,11 +140,16 @@ def _import_online(topo, no_no):
     import_task.import_suffix_from_ldif(ldiffile=import_ldif, suffix=DEFAULT_SUFFIX)
 
     # Wait a bit till the task is created and available for searching
-    time.sleep(0.5)
+    if ds_is_newer('1.4.1.2'):
+        for x in range(60):
+            if import_task.present('nstaskcreated'):
+                break
+            time.sleep(0.5)
+        assert import_task.present('nstaskcreated')
+    else:
+        time.sleep(0.5)
 
     # Good as place as any to quick test the task has some expected attributes
-    if ds_is_newer('1.4.1.2'):
-        assert import_task.present('nstaskcreated')
     assert import_task.present('nstasklog')
     assert import_task.present('nstaskcurrentitem')
     assert import_task.present('nstasktotalitems')
@@ -186,23 +216,46 @@ givenName:
     return import_ldif1
 
 
+def _now():
+    """
+    Get current time with the format that _check_for_core requires
+    """
+    now = datetime.now()
+    return now.strftime("%Y-%m-%d %H:%M:%S")
+
+
+def __check_for_core(now):
+    """
+    Check if ns-slapd generated a core since the provided date by looking in the system logs.
+    """
+    cmd = [ 'journalctl' ,'-S', now, '-t', 'audit', '-g', 'ANOM_ABEND.*ns-slapd' ]
+    result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
+    if result.returncode != 1:
+        # journalctl returns 1 if there is no matching records, and 0 if there are records
+        log.error('journalctl output is:\n%s' % result.stdout)
+        raise AssertionError(f'journalctl reported that ns-slapd crashes after {now}')
+
+
 def test_import_with_index(topo, _import_clean):
     """
     Add an index, then import via cn=tasks
 
-    :id: 5bf75c47-a283-430e-a65c-3c5fd8dbadb8
+    :id: 9ddaf0df-7298-42bb-bdfa-a889ee68bc09
     :setup: Standalone Instance
     :steps:
         1. Creating the room number index
         2. Importing online
         3. Import is done -- verifying that it worked
-    :expected results:
+    :expectedresults:
         1. Operation successful
         2. Operation successful
         3. Operation successful
     """
     place = topo.standalone.dbdir
-    assert not glob.glob(f'{place}/userRoot/roomNumber.db*', recursive=True)
+    if topo.standalone.is_dbi_supported():
+        assert not topo.standalone.is_dbi('userRoot/roomNumber.db')
+    else:
+        assert not glob.glob(f'{place}/userRoot/roomNumber.db*', recursive=True)
     # Creating the room number index
     indexes = Indexes(topo.standalone)
     indexes.create(properties={
@@ -213,21 +266,23 @@ def test_import_with_index(topo, _import_clean):
     # Importing online
     _import_online(topo, 5)
     # Import is done -- verifying that it worked
-    assert glob.glob(f'{place}/userRoot/roomNumber.db*', recursive=True)
-
+    if topo.standalone.is_dbi_supported():
+        assert topo.standalone.is_dbi('userRoot/roomNumber.db')
+    else:
+        assert glob.glob(f'{place}/userRoot/roomNumber.db*', recursive=True)
 
 
 def test_online_import_with_warning(topo, _import_clean):
     """
     Import an ldif file with syntax errors, verify skipped entry warning code
 
-    :id: 5bf75c47-a283-430e-a65c-3c5fd8dbadb8
+    :id: 9b44cd0e-9d4b-4ae9-b750-cc7ba58d4529
     :setup: Standalone Instance
     :steps:
         1. Create standalone Instance
         2. Create an ldif file with an entry that violates syntax check (empty givenname)
         3. Online import of troublesome ldif file
-    :expected results:
+    :expectedresults:
         1. Successful import with skipped entry warning
         """
     topo.standalone.restart()
@@ -245,6 +300,7 @@ def test_online_import_with_warning(topo, _import_clean):
     assert import_task.present('nstaskwarning')
     assert TaskWarning.WARN_SKIPPED_IMPORT_ENTRY == import_task.get_task_warn()
 
+
 def test_crash_on_ldif2db(topo, _import_clean):
     """
     Delete the cn=monitor entry for an LDBM backend instance. Doing this will
@@ -255,7 +311,7 @@ def test_crash_on_ldif2db(topo, _import_clean):
     :steps:
         1. Delete the cn=monitor entry for an LDBM backend instance
         2. Restart the server and verify that the LDBM monitor entry was re-created.
-    :expected results:
+    :expectedresults:
         1. Operation successful
         2. Operation successful
     """
@@ -267,7 +323,6 @@ def test_crash_on_ldif2db(topo, _import_clean):
     _import_offline(topo, 5)
 
 
-@pytest.mark.bz185477
 def test_ldif2db_allows_entries_without_a_parent_to_be_imported(topo, _import_clean):
     """Should reject import of entries that's missing parent suffix
 
@@ -276,7 +331,7 @@ def test_ldif2db_allows_entries_without_a_parent_to_be_imported(topo, _import_cl
     :steps:
         1. Import the offending LDIF data - offline
         2. Violates schema, ending line
-    :expected results:
+    :expectedresults:
         1. Operation successful
         2. Operation Fail
     """
@@ -292,6 +347,7 @@ def test_ldif2db_allows_entries_without_a_parent_to_be_imported(topo, _import_cl
 
 def test_ldif2db_syntax_check(topo, _import_clean):
     """ldif2db should return a warning when a skipped entry has occured.
+
     :id: 85e75670-42c5-4062-9edc-7f117c97a06f
     :setup:
         1. Standalone Instance
@@ -299,7 +355,7 @@ def test_ldif2db_syntax_check(topo, _import_clean):
     :steps:
         1. Create an ldif file which violates the syntax checking rule
         2. Stop the server and import ldif file with ldif2db
-    :expected results:
+    :expectedresults:
         1. ldif2db import returns a warning to signify skipped entries
     """
     import_ldif1 = _create_syntax_err_ldif(topo)
@@ -310,6 +366,7 @@ def test_ldif2db_syntax_check(topo, _import_clean):
     topo.standalone.start()
 
 
+@pytest.mark.skipif(get_default_db_lib() == "mdb", reason="Not cache size over mdb")
 def test_issue_a_warning_if_the_cache_size_is_smaller(topo, _import_clean):
     """Report during startup if nsslapd-cachememsize is too small
 
@@ -321,7 +378,7 @@ def test_issue_a_warning_if_the_cache_size_is_smaller(topo, _import_clean):
         3. Check that cachememsize is sufficiently small
         4. Import some users to make id2entry.db big
         5. Warning message should be there in error logs
-    :expected results:
+    :expectedresults:
         1. Operation successful
         2. Operation successful
         3. Operation successful
@@ -357,8 +414,10 @@ def _toggle_private_import_mem(request, topo):
             ('nsslapd-db-private-import-mem', 'off'))
     request.addfinalizer(finofaci)
 
+
 #unstable or unstatus tests, skipped for now
 #@pytest.mark.flaky(max_runs=2, min_passes=1)
+@pytest.mark.skipif(get_default_db_lib() == "mdb", reason="nsslapd-db-private-import-mem and nsslapd-import-cache-autosize parameters are ignored when usign lmdb")
 def test_fast_slow_import(topo, _toggle_private_import_mem, _import_clean):
     """With nsslapd-db-private-import-mem: on is faster import.
 
@@ -375,7 +434,7 @@ def test_fast_slow_import(topo, _toggle_private_import_mem, _import_clean):
         8. Now nsslapd-db-private-import-mem:off
         9. Measure offline import time duration total_time2
         10. total_time1 < total_time2
-    :expected results:
+    :expectedresults:
         1. Operation successful
         2. Operation successful
         3. Operation successful
@@ -418,12 +477,11 @@ def test_fast_slow_import(topo, _toggle_private_import_mem, _import_clean):
     # Measure offline import time duration total_time2
     total_time2 = _import_offline(topo, 1000)
     # total_time1 < total_time2
-    log.info("toral_time1 = %f" % total_time1)
+    log.info("total_time1 = %f" % total_time1)
     log.info("total_time2 = %f" % total_time2)
     assert total_time1 < total_time2
 
 
-@pytest.mark.bz175063
 def test_entry_with_escaped_characters_fails_to_import_and_index(topo, _import_clean):
     """If missing entry_id is found, skip it and continue reading the primary db to be re indexed.
 
@@ -434,7 +492,7 @@ def test_entry_with_escaped_characters_fails_to_import_and_index(topo, _import_c
         2. Remove some of the other entries that were successfully imported.
         3. Now re-index the database.
         4. Should not return error.
-    :expected results:
+    :expectedresults:
         1. Operation successful
         2. Operation successful
         3. Operation successful
@@ -492,16 +550,146 @@ def test_import_perf_after_failure(topo):
     time.sleep(1)
     import_task = ImportTask(topo.standalone)
     import_task.import_suffix_from_ldif(ldiffile=import_ldif, suffix=DEFAULT_SUFFIX)
-    import_task.wait(30)  # If things go wrong import takes a lot longer than this
+    import_task.wait(values['wait30'])  # If things go wrong import takes a lot longer than this
     assert import_task.is_complete()
 
     # Restart server
     topo.standalone.restart()
 
 
+def test_import_wrong_file_path(topo):
+    """Make an import fail by specifying the wrong LDIF file name
+
+    :id: 6795a3cd-b95e-4777-bc77-25ab864882a3
+    :setup: Standalone Instance
+    :steps:
+        1. Do an import with an invalid file path
+        2. Appropriate error is returned
+    :expectedresults:
+        1. Success
+        2. Success
+    """
+    import_ldif = '/nope/perf_import.ldif'
+    args = FakeArgs()
+    args.instance = topo.standalone.serverid
+    args.backend = "userroot"
+    args.encrypted = False
+    args.replication = False
+    args.ldif = import_ldif
+
+    with pytest.raises(ValueError) as e:
+        dbtasks_ldif2db(topo.standalone, log, args)
+    assert "The LDIF file does not exist" in str(e.value)
+
+
+@pytest.mark.skipif(get_default_db_lib() != "mdb", reason="lmdb specific test")
+def test_crash_on_ldif2db_with_lmdb(topo, _import_clean):
+    """Make an import fail by specifying a too small db size then check that
+    there is no crash.
+
+    :id: d42585b6-31d0-11ee-8724-482ae39447e5
+    :setup: Standalone Instance
+    :steps:
+        1. Configure a small database size
+        2. Import an ldif with 1K users
+        3. Check that ns-slapd has not aborted
+        4. Import an ldif with 500 users
+        5. Check that ns-slapd has not aborted
+    :expectedresults:
+        1. Success
+        2. Success
+        3. Success (ns-slapd should not have aborted)
+        4. Import should fail
+        5. Success (ns-slapd should not have aborted)
+
+    """
+    TINY_MAP_SIZE =  16 * 1024 * 1024
+    inst = topo.standalone
+    handler = LMDB_LDBMConfig(inst)
+    mapsize = TINY_MAP_SIZE
+    log.info(f'Set lmdb map size to {mapsize}.')
+    handler.replace('nsslapd-mdb-max-size', str(mapsize))
+    inst.stop()
+    for dbfile in ['data.mdb', 'INFO.mdb', 'lock.mdb']:
+        try:
+            os.remove(f'{inst.dbdir}/{dbfile}')
+        except FileNotFoundError:
+            pass
+    inst.start()
+    now = _now()
+    _import_offline(topo, 1000)
+    __check_for_core(now)
+    _import_clean_topo(topo)
+    with pytest.raises(AssertionError):
+        _import_offline(topo, 500_000)
+    __check_for_core(now)
+
+def test_online_import_under_load(topo):
+    """Perform an online import while the server is under load
+
+    :id: 56f7ac6f-4285-4bc4-8822-5ae9b502eee3
+    :setup: Standalone Instance
+    :steps:
+        1. Create and import LDIF for ldclt load
+        2. Start ldclt
+        3. Start online import
+        4. Wait a bit and kill the ldclt load
+        5. Check import task successfully completed
+
+    :expectedresults:
+        1. Success
+        2. Success
+        3. Success
+        4. Success
+        5. Success
+    """
+    inst = topo.standalone
+
+    # Create and import ldif to use with ldclt
+    ldif_dir = inst.get_ldif_dir()
+    import_ldif = ldif_dir + '/stress_import.ldif'
+    dbgen_users(inst, 1000, import_ldif, generic=True, suffix=DEFAULT_SUFFIX)
+    import_task = ImportTask(topo.standalone)
+    import_task.import_suffix_from_ldif(ldiffile=import_ldif,
+                                        suffix=DEFAULT_SUFFIX)
+    import_task.wait()
+    assert import_task.get_exit_code() == 0
+
+    # Start ldclt load
+    ldclt_cmd = [
+        '%s/ldclt' % inst.get_bin_dir(),
+        '-h', inst.host, '-p', str(inst.port),
+        '-f', '(uid=userXXXX)', '-e', 'esearch,random',
+        '-r1', '-R999', '-Q'
+    ]
+    p = subprocess.Popen(ldclt_cmd, start_new_session=True,
+                         stdout=subprocess.PIPE)
+    time.sleep(1)
+
+    # Start online import
+    import_task = ImportTask(topo.standalone)
+    import_task.import_suffix_from_ldif(ldiffile=import_ldif,
+                                        suffix=DEFAULT_SUFFIX)
+
+    # Wait a bit till the task is created and available for searching
+    for x in range(10):
+        if import_task.present('nstaskcreated'):
+            break
+        time.sleep(0.5)
+    assert import_task.present('nstaskcreated')
+
+    # Stop the load
+    time.sleep(3)
+    cmd = ['kill', '-9', str(p.pid)]
+    subprocess.Popen(cmd, stdout=subprocess.PIPE)
+
+    # import should finish
+    import_task.wait()
+    assert import_task.get_exit_code() == 0
+
+
 if __name__ == '__main__':
     # Run isolated
     # -s for DEBUG mode
     CURRENT_FILE = os.path.realpath(__file__)
     pytest.main("-s %s" % CURRENT_FILE)
-
