@@ -46,6 +46,19 @@ else:
     logging.getLogger(__name__).setLevel(logging.INFO)
 log = logging.getLogger(__name__)
 
+def _check_repl_changelog_backup(instance, backup_dir):
+    # Note: there is no way to check dbi on lmdb backup
+    # That said dbscan may perhaps do it ...
+    if instance.get_db_lib() is 'bdb':
+        if ds_supports_new_changelog():
+            backup_checkdir = os.path.join(backup_dir, DEFAULT_BENAME, BDB_CL_FILENAME)
+        else:
+            backup_checkdir = os.path.join(backup_dir, '.repl_changelog_backup', DEFAULT_CHANGELOG_DB)
+        if glob.glob(f'{backup_checkdir}*'):
+            log.info('Database backup is created successfully')
+        else:
+            log.fatal('test_changelog5: backup directory does not exist : {}*'.format(backup_checkdir))
+            assert False
 
 def _perform_ldap_operations(topo):
     """Add a test user, modify description, modrdn user and delete it"""
@@ -235,9 +248,6 @@ def remove_ldif_files_from_changelogdir(topo, extension):
 
 @pytest.mark.xfail(ds_is_older('1.3.10.1', '1.4.3'), reason="bug bz1685059")
 @pytest.mark.skip(reason="does not work for prefix builds")
-@pytest.mark.bz1685059
-@pytest.mark.ds50498
-@pytest.mark.bz1769296
 def test_cldump_files_removed(topo):
     """Verify bz1685059 : cl-dump generated ldif files are removed at the end, -l option is the way to keep them
 
@@ -486,15 +496,9 @@ def test_verify_changelog_online_backup(topo):
         log.fatal('test_changelog5: Online backup failed')
         assert False
 
-    if ds_supports_new_changelog():
-        backup_checkdir = os.path.join(backup_dir, DEFAULT_BENAME, BDB_CL_FILENAME)
-    else:
-        backup_checkdir = os.path.join(backup_dir, '.repl_changelog_backup', DEFAULT_CHANGELOG_DB)
-    if glob.glob(f'{backup_checkdir}*'):
-        log.info('Database backup is created successfully')
-    else:
-        log.fatal('test_changelog5: backup directory does not exist : {}*'.format(backup_checkdir))
-        assert False
+    # Note: there is no way to check dbi on lmdb backup
+    # That said dbscan may perhaps do it ...
+    _check_repl_changelog_backup(topo.ms['supplier1'], backup_dir);
 
     log.info('Run bak2db to restore directory server')
     try:
@@ -547,15 +551,7 @@ def test_verify_changelog_offline_backup(topo):
         assert False
     topo.ms['supplier1'].start()
 
-    if ds_supports_new_changelog():
-        backup_checkdir = os.path.join(backup_dir, DEFAULT_BENAME, BDB_CL_FILENAME)
-    else:
-        backup_checkdir = os.path.join(backup_dir, '.repl_changelog_backup', DEFAULT_CHANGELOG_DB)
-    if glob.glob(f'{backup_checkdir}*'):
-        log.info('Database backup is created successfully')
-    else:
-        log.fatal('test_changelog5: backup directory does not exist : {}*'.format(backup_checkdir))
-        assert False
+    _check_repl_changelog_backup(topo.ms['supplier1'], backup_dir);
 
     log.info('LDAP operations add, modify, modrdn and delete')
     _perform_ldap_operations(topo)
@@ -563,7 +559,6 @@ def test_verify_changelog_offline_backup(topo):
     _check_changelog_ldif(topo, changelog_ldif)
 
 
-@pytest.mark.ds47669
 def test_changelog_maxage(topo, changelog_init):
     """Check nsslapd-changelog max age values
 
@@ -595,7 +590,6 @@ def test_changelog_maxage(topo, changelog_init):
     add_and_check(topo, CHANGELOG, MAXAGE, 'xyz', False)
 
 
-@pytest.mark.ds47669
 def test_ticket47669_changelog_triminterval(topo, changelog_init):
     """Check nsslapd-changelog triminterval values
 
@@ -628,7 +622,6 @@ def test_ticket47669_changelog_triminterval(topo, changelog_init):
     add_and_check(topo, CHANGELOG, TRIMINTERVAL, 'xyz', False)
 
 
-@pytest.mark.ds47669
 @pytest.mark.skipif(ds_supports_new_changelog(), reason="changelog compaction is done by the backend itself, with id2entry as well, nsslapd-changelogcompactdb-interval is no longer supported")
 def test_changelog_compactdbinterval(topo, changelog_init):
     """Check nsslapd-changelog compactdbinterval values
@@ -663,7 +656,6 @@ def test_changelog_compactdbinterval(topo, changelog_init):
     add_and_check(topo, CHANGELOG, COMPACTDBINTERVAL, 'xyz', False)
 
 
-@pytest.mark.ds47669
 def test_retrochangelog_maxage(topo, changelog_init):
     """Check nsslapd-retrochangelog max age values
 
@@ -698,7 +690,6 @@ def test_retrochangelog_maxage(topo, changelog_init):
 
     topo.ms["supplier1"].log.info("ticket47669 was successfully verified.")
 
-@pytest.mark.ds50736
 def test_retrochangelog_trimming_crash(topo, changelog_init):
     """Check that when retroCL nsslapd-retrocthangelog contains invalid
     value, then the instance does not crash at shutdown
@@ -744,6 +735,37 @@ def test_retrochangelog_trimming_crash(topo, changelog_init):
     topo.ms["supplier1"].log.info("ticket 50736 was successfully verified.")
 
 
+@pytest.mark.skipif(not os.path.isfile("/usr/bin/db_stat"), reason="libdb-utils package is not installed")
+def test_changelog_pagesize(topo):
+    """Test that changelog page size is set properly
+
+    :id: 584a9a82-756d-11ed-8b38-482ae39447e5
+    :setup: Replication with two suppliers
+    :steps:
+         1. Check that file system preferred block size is 4K
+         2. Run db_stat -e -h db_home_dir
+         3. Check for 4K page size in db_stat output
+    :expectedresults:
+         1. Mark the test as skipped if block size is not 4K
+         2. Success
+         3. Should not have any 4K page size in db_stat output
+    """
+
+    s1=topo.ms["supplier1"]
+    fs_pagesize = os.statvfs(s1.ds_paths.db_home_dir).f_bsize
+    if fs_pagesize != 4096:
+        pytest.skip("This test requires that database filesystem prefered block size is 4K.")
+        return
+    try:
+        cmd = ["/usr/bin/db_stat", "-h", s1.ds_paths.db_home_dir, "-e"]
+        log.debug(f"DEBUG: Running {cmd}")
+        output = subprocess.check_output(cmd, universal_newlines=True, stderr=subprocess.STDOUT)
+    except subprocess.CalledProcessError as e:
+        self.log.error(f'Failed to gather db statistics {cmd}: "{e.output.decode()}')
+        self.log.error(e)
+        raise e
+    assert not re.match("^4096 *Page size", output, flags=re.MULTILINE)
+
 
 if __name__ == '__main__':
     # Run isolated
