@@ -44,7 +44,8 @@
         if (!(_x)) {                                                                    \
             slapi_log_err(SLAPI_LOG_ERR, "cache", "BAD CACHE ASSERTION at %s/%d: %s\n", \
                           __FILE__, __LINE__, #_x);                                     \
-            *(char *)0L = 23;                                                           \
+            slapi_log_backtrace(SLAPI_LOG_ERR);                                         \
+            *(char *)23L = 1;                                                           \
         }                                                                               \
     } while (0)
 
@@ -71,7 +72,7 @@ typedef enum {
 static void entrycache_clear_int(struct cache *cache);
 static void entrycache_set_max_size(struct cache *cache, uint64_t bytes);
 static int entrycache_remove_int(struct cache *cache, struct backentry *e);
-static void entrycache_return(struct cache *cache, struct backentry **bep);
+static void entrycache_return(struct cache *cache, struct backentry **bep, PRBool locked);
 static int entrycache_replace(struct cache *cache, struct backentry *olde, struct backentry *newe);
 static int entrycache_add_int(struct cache *cache, struct backentry *e, int state, struct backentry **alt);
 static struct backentry *entrycache_flush(struct cache *cache);
@@ -87,11 +88,11 @@ static void dncache_return(struct cache *cache, struct backdn **bdn);
 static int dncache_replace(struct cache *cache, struct backdn *olddn, struct backdn *newdn);
 static int dncache_add_int(struct cache *cache, struct backdn *bdn, int state, struct backdn **alt);
 static struct backdn *dncache_flush(struct cache *cache);
+static int cache_is_in_cache_nolock(void *ptr);
 #ifdef LDAP_CACHE_DEBUG_LRU
 static void dn_lru_verify(struct cache *cache, struct backdn *dn, int in);
 #endif
 
-
 /***** tiny hashtable implementation *****/
 
 #define HASH_VALUE(_key, _keylen) \
@@ -514,6 +515,30 @@ flush_remove_entry(struct timespec *entry_time, struct timespec *start_time)
     }
 }
 
+static inline void
+dbgec_test_if_entry_pointer_is_valid(void *e, void *prev, int slot, int line)
+{
+    /* Check if the entry pointer is rightly aligned and crash loudly otherwise */
+    if ( ((uint64_t)e) & ((sizeof(long))-1) ) {
+        /* If this message occurs, it means that we have reproduced the elusive entry cache corruption
+         * seen first while fixing replication conflict_resolution CI test 
+         * FYI some debug attempt have been stored in https://github.com/progier389/389-ds-base.git
+         * in branches:
+         *  debug-stuff1 (older try with log of debugging trick in slapd/dbgec*) 
+         *  debug-stuff2: Log the dn associated with backentries in a mmap file and retrieve the
+         *  dn of the corrupted entry. 
+         *   Note: if we are able to reproduce using this debug stuff and if it is always the same entry
+         *   we may catch the issue by adding a watchpoint when adding that backentry 
+         *   Note: you should disable the setuid to be able to use watchpoint 
+         */ 
+        slapi_log_err(SLAPI_LOG_FATAL, "dbgec_test_if_entry_pointer_is_valid", "cache.c[%d]: Wrong entry address: %p Previous entry address is: %p hash table slot is %d\n", line, e, prev, slot);
+        slapi_log_backtrace(SLAPI_LOG_FATAL);
+        *(char*)23 = 1;   /* abort() somehow corrupt gdb stack backtrace so lets generate a SIGSEGV */
+        abort();
+    }
+}
+
+
 /*
  * Flush all the cache entries that were added after the "start time"
  * This is called when a backend transaction plugin fails, and we need
@@ -531,11 +556,17 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
 {
     Hashtable *ht = cache->c_idtable; /* start with the ID table as it's in both ENTRY and DN caches */
     void *e, *laste = NULL;
+    char flush_etime[ETIME_BUFSIZ] = {0};
+    struct timespec duration;
+    struct timespec flush_start;
+    struct timespec flush_end;
 
+    clock_gettime(CLOCK_MONOTONIC, &flush_start);
     cache_lock(cache);
 
     for (size_t i = 0; i < ht->size; i++) {
         e = ht->slot[i];
+        dbgec_test_if_entry_pointer_is_valid(e, NULL, i, __LINE__);
         while (e) {
             struct backcommon *entry = (struct backcommon *)e;
             uint64_t remove_it = 0;
@@ -547,6 +578,7 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
             }
             laste = e;
             e = HASH_NEXT(ht, e);
+            dbgec_test_if_entry_pointer_is_valid(e, laste, i, __LINE__);
 
             if (remove_it) {
                 /* since we have the cache lock we know we can trust refcnt */
@@ -556,7 +588,7 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
                     lru_delete(cache, laste);
                     if (type == ENTRY_CACHE) {
                         entrycache_remove_int(cache, laste);
-                        entrycache_return(cache, (struct backentry **)&laste);
+                        entrycache_return(cache, (struct backentry **)&laste, PR_TRUE);
                     } else {
                         dncache_remove_int(cache, laste);
                         dncache_return(cache, (struct backdn **)&laste);
@@ -577,6 +609,7 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
 
         for (size_t i = 0; i < ht->size; i++) {
             e = ht->slot[i];
+            dbgec_test_if_entry_pointer_is_valid(e, NULL, i, __LINE__);
             while (e) {
                 struct backcommon *entry = (struct backcommon *)e;
                 uint64_t remove_it = 0;
@@ -588,6 +621,7 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
                 }
                 laste = e;
                 e = HASH_NEXT(ht, e);
+                dbgec_test_if_entry_pointer_is_valid(e, laste, i, __LINE__);
 
                 if (remove_it) {
                     /* since we have the cache lock we know we can trust refcnt */
@@ -596,7 +630,7 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
                         entry->ep_refcnt++;
                         lru_delete(cache, laste);
                         entrycache_remove_int(cache, laste);
-                        entrycache_return(cache, (struct backentry **)&laste);
+                        entrycache_return(cache, (struct backentry **)&laste, PR_TRUE);
                     } else {
                         /* Entry flagged for removal */
                         slapi_log_err(SLAPI_LOG_CACHE, "flush_hash",
@@ -609,6 +643,11 @@ flush_hash(struct cache *cache, struct timespec *start_time, int32_t type)
     }
 
     cache_unlock(cache);
+
+    clock_gettime(CLOCK_MONOTONIC, &flush_end);
+    slapi_timespec_diff(&flush_end, &flush_start, &duration);
+    snprintf(flush_etime, ETIME_BUFSIZ, "%" PRId64 ".%.09" PRId64 "", (int64_t)duration.tv_sec, (int64_t)duration.tv_nsec);
+    slapi_log_err(SLAPI_LOG_WARNING, "flush_hash", "Upon BETXN callback failure, entry cache is flushed during %s\n", flush_etime);
 }
 
 void
@@ -627,6 +666,7 @@ cache_init(struct cache *cache, uint64_t maxsize, int64_t maxentries, int type)
 {
     slapi_log_err(SLAPI_LOG_TRACE, "cache_init", "-->\n");
     cache->c_maxsize = maxsize;
+    /* coverity[missing_lock] */
     cache->c_maxentries = maxentries;
     cache->c_curentries = 0;
     if (config_get_slapi_counters()) {
@@ -652,7 +692,7 @@ cache_init(struct cache *cache, uint64_t maxsize, int64_t maxentries, int type)
     cache->c_lruhead = cache->c_lrutail = NULL;
     cache_make_hashes(cache, type);
 
-    if (((cache->c_mutex = slapi_pthread_mutex_alloc(PTHREAD_MUTEX_RECURSIVE)) == NULL) ||
+    if (((cache->c_mutex = PR_NewMonitor()) == NULL) ||
         ((cache->c_emutexalloc_mutex = PR_NewLock()) == NULL)) {
         slapi_log_err(SLAPI_LOG_ERR, "cache_init", "PR_NewMonitor failed\n");
         return 0;
@@ -772,7 +812,7 @@ cache_destroy_please(struct cache *cache, int type)
     slapi_counter_destroy(&cache->c_cursize);
     slapi_counter_destroy(&cache->c_hits);
     slapi_counter_destroy(&cache->c_tries);
-    slapi_pthread_mutex_free(&cache->c_mutex);
+    PR_DestroyMonitor(cache->c_mutex);
     PR_DestroyLock(cache->c_emutexalloc_mutex);
 }
 
@@ -927,7 +967,7 @@ cache_debug_hash(struct cache *cache, char **out)
     int total_entries, max_entries_per_slot, *slot_stats;
     int i, j;
     Hashtable *ht = NULL;
-    char *name = "unknown";
+    const char *name = "unknown";
 
     cache_lock(cache);
     *out = (char *)slapi_ch_malloc(1024);
@@ -1114,6 +1154,7 @@ entrycache_replace(struct cache *cache, struct backentry *olde, struct backentry
 #endif
     size_t entry_size = 0;
     struct backentry *alte = NULL;
+    Slapi_Attr *attr = NULL;
 
     LOG("=> entrycache_replace (%s) -> (%s)\n", backentry_get_ndn(olde),
         backentry_get_ndn(newe));
@@ -1129,6 +1170,14 @@ entrycache_replace(struct cache *cache, struct backentry *olde, struct backentry
 #endif
     newndn = slapi_sdn_get_ndn(backentry_get_sdn(newe));
     entry_size = cache_entry_size(newe);
+
+    /* Might have added/removed a referral */
+    if (slapi_entry_attr_find(newe->ep_entry, "ref", &attr) && attr) {
+        slapi_entry_set_flag(newe->ep_entry, SLAPI_ENTRY_FLAG_REFERRAL);
+    } else {
+        slapi_entry_clear_flag(newe->ep_entry, SLAPI_ENTRY_FLAG_REFERRAL);
+    }
+
     cache_lock(cache);
 
     /*
@@ -1149,7 +1198,7 @@ entrycache_replace(struct cache *cache, struct backentry *olde, struct backentry
     }
     /* If fails, we have to make sure the both entires are removed from the cache,
      * otherwise, we have no idea what's left in the cache or not... */
-    if (cache_is_in_cache(cache, newe)) {
+    if (cache_is_in_cache_nolock(newe)) {
         /* if we're doing a modrdn or turning an entry to a tombstone,
          * the new entry can be in the dn table already, so we need to remove that too.
          */
@@ -1244,14 +1293,14 @@ cache_return(struct cache *cache, void **ptr)
     }
     bep = *(struct backcommon **)ptr;
     if (CACHE_TYPE_ENTRY == bep->ep_type) {
-        entrycache_return(cache, (struct backentry **)ptr);
+        entrycache_return(cache, (struct backentry **)ptr, PR_FALSE);
     } else if (CACHE_TYPE_DN == bep->ep_type) {
         dncache_return(cache, (struct backdn **)ptr);
     }
 }
 
 static void
-entrycache_return(struct cache *cache, struct backentry **bep)
+entrycache_return(struct cache *cache, struct backentry **bep, PRBool locked)
 {
     struct backentry *eflush = NULL;
     struct backentry *eflushtemp = NULL;
@@ -1265,7 +1314,9 @@ entrycache_return(struct cache *cache, struct backentry **bep)
     LOG("entrycache_return - (%s) entry count: %d, entry in cache:%ld\n",
         backentry_get_ndn(e), e->ep_refcnt, cache->c_curentries);
 
-    cache_lock(cache);
+    if (locked == PR_FALSE) {
+        cache_lock(cache);
+    }
     if (e->ep_state & ENTRY_STATE_NOTINCACHE) {
         backentry_free(bep);
     } else {
@@ -1299,7 +1350,9 @@ entrycache_return(struct cache *cache, struct backentry **bep)
             }
         }
     }
-    cache_unlock(cache);
+    if (locked == PR_FALSE) {
+        cache_unlock(cache);
+    }
     while (eflush) {
         eflushtemp = BACK_LRU_NEXT(eflush, struct backentry *);
         backentry_free(&eflush);
@@ -1419,6 +1472,7 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state, struct b
     struct backentry *my_alt;
     size_t entry_size = 0;
     int already_in = 0;
+    Slapi_Attr *attr = NULL;
 
     LOG("=> entrycache_add_int( \"%s\", %ld )\n", backentry_get_ndn(e),
         (long int)e->ep_id);
@@ -1432,6 +1486,13 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state, struct b
     } else {
         entry_size = e->ep_size;
     }
+    /* Check for referrals now so we don't have to do it for every base
+     * search in the future */
+    if (slapi_entry_attr_find(e->ep_entry, "ref", &attr) && attr) {
+        slapi_entry_set_flag(e->ep_entry, SLAPI_ENTRY_FLAG_REFERRAL);
+    } else {
+        slapi_entry_clear_flag(e->ep_entry, SLAPI_ENTRY_FLAG_REFERRAL);
+    }
 
     cache_lock(cache);
     if (!add_hash(cache->c_dntable, (void *)ndn, strlen(ndn), e,
@@ -1628,13 +1689,13 @@ cache_add_tentative(struct cache *cache, struct backentry *e, struct backentry *
 void
 cache_lock(struct cache *cache)
 {
-    pthread_mutex_lock(cache->c_mutex);
+    PR_EnterMonitor(cache->c_mutex);
 }
 
 void
 cache_unlock(struct cache *cache)
 {
-    pthread_mutex_unlock(cache->c_mutex);
+    PR_ExitMonitor(cache->c_mutex);
 }
 
 /* locks an entry so that it can be modified (you should have gotten the
@@ -1683,6 +1744,25 @@ cache_lock_entry(struct cache *cache, struct backentry *e)
     return 0;
 }
 
+int
+cache_is_reverted_entry(struct cache *cache, struct backentry *e)
+{
+    struct backentry *dummy_e;
+
+    cache_lock(cache);
+    if (find_hash(cache->c_idtable, &e->ep_id, sizeof(ID), (void **)&dummy_e)) {
+        if (dummy_e->ep_state & ENTRY_STATE_INVALID) {
+            slapi_log_err(SLAPI_LOG_WARNING, "cache_is_reverted_entry", "Entry reverted = %d (0x%lX)  [entry: %p] refcnt=%d\n",
+                          dummy_e->ep_state,
+                          pthread_self(),
+                          dummy_e, dummy_e->ep_refcnt);
+            cache_unlock(cache);
+            return 1;
+        }
+    }
+    cache_unlock(cache);
+    return 0;
+}
 /* the opposite of above */
 void
 cache_unlock_entry(struct cache *cache __attribute__((unused)), struct backentry *e)
@@ -2143,7 +2223,7 @@ check_entry_cache(struct cache *cache, struct backentry *e)
     struct backentry *debug_e = cache_find_dn(cache,
                                               slapi_sdn_get_dn(sdn),
                                               slapi_sdn_get_ndn_len(sdn));
-    in_cache = cache_is_in_cache(cache, (void *)e);
+    int in_cache = cache_is_in_cache(cache, (void *)e);
     if (in_cache) {
         if (debug_e) { /* e is in cache */
             CACHE_RETURN(cache, &debug_e);
@@ -2210,8 +2290,8 @@ cache_has_otherref(struct cache *cache, void *ptr)
     return (hasref > 1) ? 1 : 0;
 }
 
-int
-cache_is_in_cache(struct cache *cache, void *ptr)
+static int
+cache_is_in_cache_nolock(void *ptr)
 {
     struct backcommon *bep;
     int in_cache = 0;
@@ -2220,8 +2300,16 @@ cache_is_in_cache(struct cache *cache, void *ptr)
         return in_cache;
     }
     bep = (struct backcommon *)ptr;
-    cache_lock(cache);
     in_cache = (bep->ep_state & (ENTRY_STATE_DELETED | ENTRY_STATE_NOTINCACHE)) ? 0 : 1;
-    cache_unlock(cache);
     return in_cache;
 }
+
+int
+cache_is_in_cache(struct cache *cache, void *ptr)
+{
+    int ret;
+    cache_lock(cache);
+    ret = cache_is_in_cache_nolock(ptr);
+    cache_unlock(cache);
+    return ret;
+}
