@@ -185,7 +185,7 @@ do_bind(Slapi_PBlock *pb)
             static char *kmsg =
                 "LDAPv2-style kerberos authentication received "
                 "on LDAPv3 connection.";
-            slapi_log_err(SLAPI_LOG_ERR, "do_bind", "%s", kmsg);
+            slapi_log_err(SLAPI_LOG_ERR, "do_bind", "%s\n", kmsg);
             log_bind_access(pb, dn ? dn : "empty", method, version, saslmech, kmsg);
             send_ldap_result(pb, LDAP_PROTOCOL_ERROR, NULL,
                              kmsg, 0, NULL);
@@ -349,9 +349,9 @@ do_bind(Slapi_PBlock *pb)
             while (1) {
                 if (*pmech == NULL) {
                     /*
-                 * As we call the safe function, we receive a strdup'd saslmechanisms
-                 * charray. Therefore, we need to remove it instead of NULLing it
-                 */
+                     * As we call the safe function, we receive a strdup'd saslmechanisms
+                     * charray. Therefore, we need to remove it instead of NULLing it
+                     */
                     charray_free(supported);
                     pmech = supported = NULL;
                     break;
@@ -415,8 +415,12 @@ do_bind(Slapi_PBlock *pb)
             if (NULL == pb_conn->c_external_dn) {
                 slapi_pblock_set(pb, SLAPI_PB_RESULT_TEXT, "Client certificate mapping failed");
                 send_ldap_result(pb, LDAP_INVALID_CREDENTIALS, NULL, "", 0, NULL);
+                slapi_log_security(pb, SECURITY_BIND_FAILED, SECURITY_MSG_CERT_MAP_FAILED);
                 /* call postop plugins */
                 plugin_call_plugins(pb, SLAPI_PLUGIN_POST_BIND_FN);
+                if (config_get_close_on_failed_bind()) {
+                    disconnect_server_nomutex(pb_conn, pb_op->o_connid, pb_op->o_opid, SLAPD_DISCONNECT_UNBIND, 0);
+                }
                 goto free_and_return;
             }
 
@@ -441,6 +445,7 @@ do_bind(Slapi_PBlock *pb)
                 slapi_add_auth_response_control(pb, pb_conn->c_external_dn);
             }
             send_ldap_result(pb, LDAP_SUCCESS, NULL, NULL, 0, NULL);
+            slapi_log_security(pb, SECURITY_BIND_SUCCESS, "");
             /* call postop plugins */
             plugin_call_plugins(pb, SLAPI_PLUGIN_POST_BIND_FN);
             goto free_and_return;
@@ -495,7 +500,7 @@ do_bind(Slapi_PBlock *pb)
                     slapi_add_auth_response_control(pb, "");
                 }
                 send_ldap_result(pb, LDAP_SUCCESS, NULL, NULL, 0, NULL);
-
+                slapi_log_security(pb, SECURITY_BIND_SUCCESS, SECURITY_MSG_ANONYMOUS_BIND);
                 /* call postop plugins */
                 plugin_call_plugins(pb, SLAPI_PLUGIN_POST_BIND_FN);
             } else {
@@ -602,9 +607,13 @@ do_bind(Slapi_PBlock *pb)
                  */
                 slapi_pblock_set(pb, SLAPI_PB_RESULT_TEXT, "Invalid credentials");
                 send_ldap_result(pb, LDAP_INVALID_CREDENTIALS, NULL, NULL, 0, NULL);
+                slapi_log_security(pb, SECURITY_BIND_FAILED, SECURITY_MSG_INVALID_PASSWD);
                 /* increment BindSecurityErrorcount */
                 slapi_counter_increment(g_get_per_thread_snmp_vars()->ops_tbl.dsBindSecurityErrors);
                 value_done(&cv);
+                if (config_get_close_on_failed_bind()) {
+                    disconnect_server_nomutex(pb_conn, pb_op->o_connid, pb_op->o_opid, SLAPD_DISCONNECT_UNBIND, 0);
+                }
                 goto free_and_return;
             }
 
@@ -618,7 +627,7 @@ do_bind(Slapi_PBlock *pb)
                                                 (cred.bv_len == 0) ? "" : slapi_sdn_get_ndn(sdn));
             }
             send_ldap_result(pb, LDAP_SUCCESS, NULL, NULL, 0, NULL);
-
+            slapi_log_security(pb, SECURITY_BIND_SUCCESS, "");
             /* call postop plugins */
             plugin_call_plugins(pb, SLAPI_PLUGIN_POST_BIND_FN);
         } else {
@@ -742,6 +751,10 @@ do_bind(Slapi_PBlock *pb)
                     myrc = slapi_check_account_lock(pb, bind_target_entry, pw_response_requested, 1, 1);
                     if (1 == myrc) { /* account is locked */
                         rc = myrc;
+                        slapi_log_security(pb, SECURITY_BIND_FAILED, SECURITY_MSG_ACCOUNT_LOCKED);
+                        if (config_get_close_on_failed_bind()) {
+                            disconnect_server_nomutex(pb_conn, pb_op->o_connid, pb_op->o_opid, SLAPD_DISCONNECT_UNBIND, 0);
+                        }
                         goto account_locked;
                     }
                     myrc = 0;
@@ -790,6 +803,10 @@ do_bind(Slapi_PBlock *pb)
                         if (!bind_target_entry) {
                             slapi_pblock_set(pb, SLAPI_PB_RESULT_TEXT, "No such entry");
                             send_ldap_result(pb, LDAP_INVALID_CREDENTIALS, NULL, "", 0, NULL);
+                            slapi_log_security(pb, SECURITY_BIND_FAILED, SECURITY_MSG_NO_ENTRY);
+                            if (config_get_close_on_failed_bind()) {
+                                disconnect_server_nomutex(pb_conn, pb_op->o_connid, pb_op->o_opid, SLAPD_DISCONNECT_UNBIND, 0);
+                            }
                             goto free_and_return;
                         }
                     }
@@ -838,6 +855,15 @@ do_bind(Slapi_PBlock *pb)
             if (rc == LDAP_OPERATIONS_ERROR) {
                 send_ldap_result(pb, LDAP_UNWILLING_TO_PERFORM, NULL, "Function not implemented", 0, NULL);
                 goto free_and_return;
+            } else {
+                if (bind_target_entry) {
+                    slapi_log_security(pb, SECURITY_BIND_FAILED, SECURITY_MSG_INVALID_PASSWD);
+                } else {
+                    slapi_log_security(pb, SECURITY_BIND_FAILED, SECURITY_MSG_NO_ENTRY);
+                }
+                if (config_get_close_on_failed_bind()) {
+                    disconnect_server_nomutex(pb_conn, pb_op->o_connid, pb_op->o_opid, SLAPD_DISCONNECT_UNBIND, 0);
+                }
             }
         account_locked:
             if (cred.bv_len == 0) {
@@ -861,6 +887,7 @@ do_bind(Slapi_PBlock *pb)
          */
         if (rc == SLAPI_BIND_SUCCESS || rc == SLAPI_BIND_ANONYMOUS) {
             send_ldap_result(pb, LDAP_SUCCESS, NULL, NULL, 0, NULL);
+            slapi_log_security(pb, SECURITY_BIND_SUCCESS, "");
         }
 
         slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, &rc);
@@ -909,34 +936,73 @@ log_bind_access(
     Connection *pb_conn = NULL;
     slapi_pblock_get(pb, SLAPI_OPERATION, &pb_op);
     slapi_pblock_get(pb, SLAPI_CONNECTION, &pb_conn);
+    int32_t log_format = config_get_accesslog_log_format();
+    slapd_log_pblock logpb = {0};
 
     if (pb_op == NULL || pb_conn == NULL) {
         return;
     }
 
+    slapd_log_pblock_init(&logpb, log_format, pb);
+    logpb.version = version;
+    logpb.bind_dn = dn;
+    logpb.method = "sasl";
+
     if (method == LDAP_AUTH_SASL && saslmech && msg) {
-        slapi_log_access(LDAP_DEBUG_STATS,
-                         "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
-                         "method=sasl version=%d mech=%s, %s\n",
-                         pb_conn->c_connid, pb_op->o_opid, dn,
-                         version, saslmech, msg);
+        if (log_format != LOG_FORMAT_DEFAULT) {
+            /* JSON logging */
+            logpb.mech = saslmech;
+            logpb.msg = msg;
+            logpb.method = "sasl";
+            slapd_log_access_bind(&logpb);
+        } else {
+            slapi_log_access(LDAP_DEBUG_STATS,
+                             "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
+                             "method=sasl version=%d mech=%s, %s\n",
+                             pb_conn->c_connid, pb_op->o_opid, dn,
+                             version, saslmech, msg);
+        }
     } else if (method == LDAP_AUTH_SASL && saslmech) {
-        slapi_log_access(LDAP_DEBUG_STATS,
-                         "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
-                         "method=sasl version=%d mech=%s\n",
-                         pb_conn->c_connid, pb_op->o_opid, dn,
-                         version, saslmech);
+        if (log_format != LOG_FORMAT_DEFAULT) {
+            /* JSON logging */
+            logpb.mech = saslmech;
+            logpb.method = "sasl";
+            slapd_log_access_bind(&logpb);
+        } else {
+            slapi_log_access(LDAP_DEBUG_STATS,
+                             "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
+                             "method=sasl version=%d mech=%s\n",
+                             pb_conn->c_connid, pb_op->o_opid, dn,
+                             version, saslmech);
+        }
     } else if (msg) {
-        slapi_log_access(LDAP_DEBUG_STATS,
-                         "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
-                         "method=%" BERTAG_T " version=%d, %s\n",
-                         pb_conn->c_connid, pb_op->o_opid, dn,
-                         method, version, msg);
+        if (log_format != LOG_FORMAT_DEFAULT) {
+            /* JSON logging */
+            logpb.mech = saslmech;
+            logpb.msg = msg;
+            logpb.method = "sasl";
+            slapd_log_access_bind(&logpb);
+        } else {
+            slapi_log_access(LDAP_DEBUG_STATS,
+                             "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
+                             "method=%" BERTAG_T " version=%d, %s\n",
+                             pb_conn->c_connid, pb_op->o_opid, dn,
+                             method, version, msg);
+        }
     } else {
-        slapi_log_access(LDAP_DEBUG_STATS,
-                         "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
-                         "method=%" BERTAG_T " version=%d\n",
-                         pb_conn->c_connid, pb_op->o_opid, dn,
-                         method, version);
+        if (log_format != LOG_FORMAT_DEFAULT) {
+            /* JSON logging */
+            char method_str[22] = {0};
+
+            PR_snprintf(method_str, sizeof(method_str), "%lu", method);
+            logpb.method = method_str;
+            slapd_log_access_bind(&logpb);
+        } else {
+            slapi_log_access(LDAP_DEBUG_STATS,
+                             "conn=%" PRIu64 " op=%d BIND dn=\"%s\" "
+                             "method=%" BERTAG_T " version=%d\n",
+                             pb_conn->c_connid, pb_op->o_opid, dn,
+                             method, version);
+        }
     }
 }
