@@ -72,6 +72,9 @@ init_controls(void)
                                          SLAPI_OPERATION_MODDN);
     slapi_register_supported_control(LDAP_X_CONTROL_PWPOLICY_REQUEST,
                                      SLAPI_OPERATION_SEARCH | SLAPI_OPERATION_COMPARE | SLAPI_OPERATION_ADD | SLAPI_OPERATION_DELETE | SLAPI_OPERATION_MODIFY | SLAPI_OPERATION_MODDN);
+    slapi_register_supported_control(LDAP_CONTROL_SUBENTRIES,
+                                     SLAPI_OPERATION_SEARCH);
+
     /*
     We do not register the password policy response because it has
     the same oid as the request (and it was being reported twice in
@@ -203,6 +206,7 @@ get_ldapmessage_controls_ext(
     }
 
     ctrls = NULL;
+    /* coverity[var_deref_model] */
     slapi_pblock_set(pb, SLAPI_REQCONTROLS, ctrls);
     if (controlsp != NULL) {
         *controlsp = NULL;
@@ -640,3 +644,14 @@ slapi_build_control_from_berval(char *oid, struct berval *bvp, char iscritical,
 
     return return_value;
 }
+
+/*
+ * Parse an LDAP control into its parts
+ */
+void
+slapi_parse_control(LDAPControl *ctrl, char **oid, char **value, PRBool *isCritical)
+{
+    *isCritical = ctrl->ldctl_iscritical;
+    *value = ctrl->ldctl_value.bv_val;
+    *oid = ctrl->ldctl_oid;
+}
