@@ -1,6 +1,6 @@
 /** BEGIN COPYRIGHT BLOCK
  * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
- * Copyright (C) 2005 Red Hat, Inc.
+ * Copyright (C) 2021 Red Hat, Inc.
  * All rights reserved.
  *
  * License: GPL (version 3 or any later version).
@@ -443,7 +443,7 @@ csngen_test()
     int rc;
     CSNGen *gen = csngen_new(255, NULL);
 
-    slapi_log_err(SLAPI_LOG_DEBUG, "csngen_test", "staring csn generator test ...");
+    slapi_log_err(SLAPI_LOG_DEBUG, "csngen_test", "staring csn generator test ...\n");
     csngen_dump_state(gen, SLAPI_LOG_INFO);
 
     rc = _csngen_start_test_threads(gen);
@@ -455,7 +455,7 @@ csngen_test()
 
     _csngen_stop_test_threads();
     csngen_dump_state(gen, SLAPI_LOG_INFO);
-    slapi_log_err(SLAPI_LOG_DEBUG, "csngen_test", "csn generator test is complete...");
+    slapi_log_err(SLAPI_LOG_DEBUG, "csngen_test", "csn generator test is complete...\n");
 }
 
 /*
@@ -840,12 +840,12 @@ int32_t _csngen_tester_gettime(struct timespec *tp)
     return 0;
 }
 
-/* Mimic a fully meshed multi suplier topology */
+/* Mimic a fully meshed multi supplier topology */
 void csngen_multi_suppliers_test(void)
 {
-#define NB_TEST_MASTERS	6
+#define NB_TEST_SUPPLIERS	6
 #define NB_TEST_STATES	500
-    CSNGen *gen[NB_TEST_MASTERS];
+    CSNGen *gen[NB_TEST_SUPPLIERS];
     struct timespec now = {0};
     CSN *last_csn = NULL;
     CSN *csn = NULL;
@@ -853,14 +853,16 @@ void csngen_multi_suppliers_test(void)
 
     _csngen_tester_gettime(&now);
 
-    for (i=0; i< NB_TEST_MASTERS; i++) {
+    for (i=0; i< NB_TEST_SUPPLIERS; i++) {
         gen[i] = csngen_new(i+1, NULL);
-        gen[i]->gettime = _csngen_tester_gettime;
-        gen[i]->state.sampled_time = now.tv_sec;
+        if (gen[i]) {
+            gen[i]->gettime = _csngen_tester_gettime;
+            gen[i]->state.sampled_time = now.tv_sec;
+        }
     }
 
     for (_csngen_tester_state=0; _csngen_tester_state < NB_TEST_STATES; _csngen_tester_state++) {
-        for (i=0; i< NB_TEST_MASTERS; i++) {
+        for (i=0; i< NB_TEST_SUPPLIERS; i++) {
             _csngen_tester_state_rid = i+1;
             csn = NULL;
             rc = csngen_new_csn(gen[i], &csn, PR_FALSE);
@@ -880,7 +882,7 @@ void csngen_multi_suppliers_test(void)
             csn_free(&last_csn);
             last_csn = csn;
 
-            for (j=0; j< NB_TEST_MASTERS; j++) {
+            for (j=0; j< NB_TEST_SUPPLIERS; j++) {
                 if (i==j) {
                     continue;
                 }
