@@ -219,6 +219,7 @@ cache_return_target_entry(Slapi_PBlock *pb, Slapi_Backend *be, Slapi_Operation *
         operation_set_target_entry_id(operation, 0);
     }
 }
+
 /*
  * Returns: 0    - if the operation is successful
  *        < 0    - if operation fails.
@@ -250,7 +251,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
     char *errtext = NULL;
     int nentries, pnentries;
     int flag_search_base_found = 0;
-    int flag_no_such_object = 0;
+    bool flag_no_such_object = false;
     int flag_referral = 0;
     int flag_psearch = 0;
     int err_code = LDAP_SUCCESS;
@@ -262,7 +263,6 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
     int sent_result = 0;
     unsigned int pr_stat = 0;
     Connection *pb_conn;
-
     ber_int_t pagesize = -1;
     ber_int_t estimate = 0; /* estimated search result set size */
     int curr_search_count = 0;
@@ -271,6 +271,9 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
     int pr_idx = -1;
     Slapi_DN *orig_sdn = NULL;
     int free_sdn = 0;
+    pthread_mutex_t *pagedresults_mutex = NULL;
+    int32_t log_format = config_get_accesslog_log_format();
+    slapd_log_pblock logpb = {0};
 
     be_list[0] = NULL;
     referral_list[0] = NULL;
@@ -314,7 +317,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
         rc = -1;
         goto free_and_return_nolock;
     }
-    
+
     /* Set the time we actually started the operation */
     slapi_operation_set_time_started(operation);
 
@@ -331,10 +334,12 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
         int32_t op_id;
         int32_t op_internal_id;
         int32_t op_nested_count;
+        time_t start_time;
 
         PR_ASSERT(fstr);
-        if (internal_op) {
-            get_internal_conn_op(&connid, &op_id, &op_internal_id, &op_nested_count);
+
+        if (proxydn) {
+            proxystr = slapi_ch_smprintf(" authzid=\"%s\"", proxydn);
         }
 
         if (NULL == attrs) {
@@ -345,10 +350,6 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
             attrliststr = attrlistbuf;
         }
 
-        if (proxydn) {
-            proxystr = slapi_ch_smprintf(" authzid=\"%s\"", proxydn);
-        }
-
 #define SLAPD_SEARCH_FMTSTR_CONN_OP "conn=%" PRIu64 " op=%d"
 #define SLAPD_SEARCH_FMTSTR_CONN_OP_INT_INT "conn=Internal(%" PRIu64 ") op=%d(%d)(%d)"
 #define SLAPD_SEARCH_FMTSTR_CONN_OP_EXT_INT "conn=%" PRIu64 " (Internal) op=%d(%d)(%d)"
@@ -369,6 +370,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
         if (!internal_op) {
             strcpy(fmtstr, SLAPD_SEARCH_FMTSTR_CONN_OP);
         } else {
+            get_internal_conn_op(&connid, &op_id, &op_internal_id, &op_nested_count, &start_time);
             if (connid == 0) {
                 strcpy(fmtstr, SLAPD_SEARCH_FMTSTR_CONN_OP_INT_INT);
             } else {
@@ -380,24 +382,48 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
         strcat(fmtstr, LOG_ACCESS_FORMAT_ATTR_BUFSIZ(attrliststr, "\" attrs=", SLAPD_SEARCH_BUFPART));
         strcat(fmtstr, SLAPD_SEARCH_FMTSTR_REMAINDER);
 
+        /* Prep log pblock */
+        slapd_log_pblock_init(&logpb, log_format, pb);
+        logpb.authzid = proxydn;
+        logpb.base_dn = normbase;
+        logpb.scope = scope;
+        logpb.filter = fstr;
+        logpb.attrs = attrs;
+        logpb.psearch = flag_psearch ? PR_TRUE : PR_FALSE;
+
         if (!internal_op) {
-            slapi_log_access(LDAP_DEBUG_STATS, fmtstr,
-                             pb_conn->c_connid,
-                             operation->o_opid,
-                             normbase,
-                             scope, fstr, attrliststr,
-                             flag_psearch ? " options=persistent" : "",
-                             proxystr ? proxystr : "");
+            if (log_format != LOG_FORMAT_DEFAULT) {
+                slapd_log_access_search(&logpb);
+            } else {
+                slapi_log_access(LDAP_DEBUG_STATS, fmtstr,
+                                 pb_conn->c_connid,
+                                 operation->o_opid,
+                                 normbase,
+                                 scope, fstr, attrliststr,
+                                 flag_psearch ? " options=persistent" : "",
+                                 proxystr ? proxystr : "");
+            }
+
         } else {
-            slapi_log_access(LDAP_DEBUG_ARGS, fmtstr,
-                             connid,
-                             op_id,
-                             op_internal_id,
-                             op_nested_count,
-                             normbase,
-                             scope, fstr, attrliststr,
-                             flag_psearch ? " options=persistent" : "",
-                             proxystr ? proxystr : "");
+            /* Internal op */
+            if (log_format != LOG_FORMAT_DEFAULT) {
+                logpb.conn_time = start_time;
+                logpb.conn_id = connid;
+                logpb.op_id = op_id;
+                logpb.op_internal_id = op_internal_id;
+                logpb.op_nested_count = op_nested_count;
+                slapd_log_access_search(&logpb);
+            } else {
+                slapi_log_access(LDAP_DEBUG_ARGS, fmtstr,
+                                 connid,
+                                 op_id,
+                                 op_internal_id,
+                                 op_nested_count,
+                                 normbase,
+                                 scope, fstr, attrliststr,
+                                 flag_psearch ? " options=persistent" : "",
+                                 proxystr ? proxystr : "");
+            }
         }
     }
 
@@ -480,6 +506,20 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
             while (be_list[index] && be_list[index + 1]) {
                 index++;
             }
+            if (scope == LDAP_SCOPE_ONELEVEL) {
+                /*
+                 * ONE LEVEL searches may ends up on multiple backends
+                 *  with a ONE LEVEL search on a suffix and a BASE search on its
+                 *  subsuffixes. Because LDAP_SCOPE_ONELEVEL rewrite the filter
+                 *  the backends should be reversed so that the BASE search(es)
+                 *  are done first (with the original filter).
+                 */
+                for (int idx = 0; idx <= index/2; idx++) {
+                    be = be_list[index-idx];
+                    be_list[index-idx] = be_list[idx];
+                    be_list[idx] = be;
+                }
+            }
             be = be_list[index];
         } else {
             be = NULL;
@@ -576,6 +616,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
             int32_t tlimit;
             slapi_pblock_get(pb, SLAPI_SEARCH_TIMELIMIT, &tlimit);
             pagedresults_set_timelimit(pb_conn, operation, (time_t)tlimit, pr_idx);
+            pagedresults_mutex = pageresult_lock_get_addr(pb_conn);
         }
 
         /*
@@ -696,7 +737,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
              * In async paged result case, the search result might be released
              * by other theads.  We need to double check it in the locked region.
              */
-            pthread_mutex_lock(&(pb_conn->c_mutex));
+            pthread_mutex_lock(pagedresults_mutex);
             pr_search_result = pagedresults_get_search_result(pb_conn, operation, 1 /*locked*/, pr_idx);
             if (pr_search_result) {
                 if (pagedresults_is_abandoned_or_notavailable(pb_conn, 1 /*locked*/, pr_idx)) {
@@ -704,7 +745,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                     /* Previous operation was abandoned and the simplepaged object is not in use. */
                     send_ldap_result(pb, 0, NULL, "Simple Paged Results Search abandoned", 0, NULL);
                     rc = LDAP_SUCCESS;
-                    pthread_mutex_unlock(&(pb_conn->c_mutex));
+                    pthread_mutex_unlock(pagedresults_mutex);
                     goto free_and_return;
                 } else {
                     slapi_pblock_set(pb, SLAPI_SEARCH_RESULT_SET, pr_search_result);
@@ -718,7 +759,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                 pr_stat = PAGEDRESULTS_SEARCH_END;
                 rc = LDAP_SUCCESS;
             }
-            pthread_mutex_unlock(&(pb_conn->c_mutex));
+            pthread_mutex_unlock(pagedresults_mutex);
             pagedresults_unlock(pb_conn, pr_idx);
 
             if ((PAGEDRESULTS_SEARCH_END == pr_stat) || (0 == pnentries)) {
@@ -777,7 +818,6 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                         (slapi_sdn_get_ndn_len(basesdn) == 0)) {
                         int tmp_scope = LDAP_SCOPE_BASE;
                         slapi_pblock_set(pb, SLAPI_SEARCH_SCOPE, &tmp_scope);
-
                         if (free_sdn) {
                             slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
                             slapi_sdn_free(&sdn);
@@ -788,6 +828,12 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                     } else if (slapi_sdn_issuffix(basesdn, be_suffix)) {
                         int tmp_scope = LDAP_SCOPE_ONELEVEL;
                         slapi_pblock_set(pb, SLAPI_SEARCH_SCOPE, &tmp_scope);
+                        if (free_sdn) {
+                            slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
+                            slapi_sdn_free(&sdn);
+                            sdn = slapi_sdn_dup(basesdn);
+                            slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, (void *)sdn);
+                        }
                     } else {
                         slapi_sdn_done(&monitorsdn);
                         goto next_be;
@@ -830,7 +876,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                     /* may be the object exist somewhere else
                      * wait the end of the loop to send back this error
                      */
-                    flag_no_such_object = 1;
+                    flag_no_such_object = true;
                 } else {
                     /* err something other than LDAP_NO_SUCH_OBJECT, so the backend will
                      * have sent the result -
@@ -840,35 +886,35 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                 /* fall through */
 
             case -1: /* an error occurred */
+                slapi_pblock_get(pb, SLAPI_RESULT_CODE, &err);
                 /* PAGED RESULTS */
                 if (op_is_pagedresults(operation)) {
                     /* cleanup the slot */
-                    pthread_mutex_lock(&(pb_conn->c_mutex));
+                    pthread_mutex_lock(pagedresults_mutex);
+                    if (err != LDAP_NO_SUCH_OBJECT && !flag_no_such_object) {
+                        /* Free the results if not "no_such_object" */
+                        void *sr = NULL;
+                        slapi_pblock_get(pb, SLAPI_SEARCH_RESULT_SET, &sr);
+                        be->be_search_results_release(&sr);
+                    }
                     pagedresults_set_search_result(pb_conn, operation, NULL, 1, pr_idx);
                     rc = pagedresults_set_current_be(pb_conn, NULL, pr_idx, 1);
-                    pthread_mutex_unlock(&(pb_conn->c_mutex));
+                    pthread_mutex_unlock(pagedresults_mutex);
                 }
-                if (1 == flag_no_such_object) {
-                    break;
-                }
-                slapi_pblock_get(pb, SLAPI_RESULT_CODE, &err);
-                if (err == LDAP_NO_SUCH_OBJECT) {
-                    /* may be the object exist somewhere else
-                     * wait the end of the loop to send back this error
-                     */
-                    flag_no_such_object = 1;
+
+                if (err == LDAP_NO_SUCH_OBJECT || flag_no_such_object) {
+                    /* Maybe the object exists somewhere else, wait to the end
+                     * of the loop to send back this error */
+                    flag_no_such_object = true;
                     break;
                 } else {
-                    /* for error other than LDAP_NO_SUCH_OBJECT
-                     * the error has already been sent
-                     * stop the search here
-                     */
+                    /* For error other than LDAP_NO_SUCH_OBJECT the error has
+                     * already been sent stop the search here */
                     cache_return_target_entry(pb, be, operation);
                     goto free_and_return;
                 }
 
             /* when rc == SLAPI_FAIL_DISKFULL this case is executed */
-
             case SLAPI_FAIL_DISKFULL:
                 operation_out_of_disk_space();
                 cache_return_target_entry(pb, be, operation);
@@ -887,11 +933,11 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                     slapi_pblock_get(pb, SLAPI_SEARCH_RESULT_SET, &sr);
                     if ((PAGEDRESULTS_SEARCH_END == pr_stat) || (0 == pnentries)) {
                         /* no more entries, but at least another backend */
-                        pthread_mutex_lock(&(pb_conn->c_mutex));
+                        pthread_mutex_lock(pagedresults_mutex);
                         pagedresults_set_search_result(pb_conn, operation, NULL, 1, pr_idx);
                         be->be_search_results_release(&sr);
                         rc = pagedresults_set_current_be(pb_conn, next_be, pr_idx, 1);
-                        pthread_mutex_unlock(&(pb_conn->c_mutex));
+                        pthread_mutex_unlock(pagedresults_mutex);
                         pr_stat = PAGEDRESULTS_SEARCH_END; /* make sure stat is SEARCH_END */
                         if (NULL == next_be) {
                             /* no more entries && no more backends */
@@ -919,9 +965,7 @@ op_shared_search(Slapi_PBlock *pb, int send_result)
                     next_be = NULL; /* to break the loop */
                     if (operation->o_status & SLAPI_OP_STATUS_ABANDONED) {
                         /* It turned out this search was abandoned. */
-                        pthread_mutex_lock(&(pb_conn->c_mutex));
-                        pagedresults_free_one_msgid_nolock(pb_conn, operation->o_msgid);
-                        pthread_mutex_unlock(&(pb_conn->c_mutex));
+                        pagedresults_free_one_msgid(pb_conn, operation->o_msgid, pagedresults_mutex);
                         /* paged-results-request was abandoned; making an empty cookie. */
                         pagedresults_set_response_control(pb, 0, estimate, -1, pr_idx);
                         send_ldap_result(pb, 0, NULL, "Simple Paged Results Search abandoned", 0, NULL);
@@ -1040,6 +1084,10 @@ free_and_return:
     } else if (be_single) {
         slapi_be_Unlock(be_single);
     }
+    if (rc > 0) {
+        /* rc > 0 means a plugin generated an error and we should abort */
+        send_ldap_result(pb, rc, NULL, NULL, 0, NULL);
+    }
 
 free_and_return_nolock:
     slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, &rc);
@@ -1049,6 +1097,10 @@ free_and_return_nolock:
         slapi_sdn_free(&sdn);
     }
     slapi_sdn_free(&basesdn);
+    /* coverity false positive:
+     *  var_deref_model: Passing null pointer "orig_sdn" to "slapi_pblock_set", which dereferences it.
+     */
+    /* coverity[var_deref_model] */
     slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, orig_sdn);
 
     slapi_ch_free_string(&proxydn);
@@ -1126,27 +1178,27 @@ process_entry(Slapi_PBlock *pb, Slapi_Entry *e, int send_result)
 static void
 send_entry(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Operation *operation, char **attrs, int attrsonly, int *pnentries)
 {
-                /*
-             * It's a regular entry, or it's a referral and
-             * managedsait control is on.  In either case, send
-             * the entry.
-             */
-                switch (send_ldap_search_entry(pb, e, NULL, attrs, attrsonly)) {
-                case 0: /* entry sent ok */
-                    (*pnentries)++;
-                    slapi_pblock_set(pb, SLAPI_NENTRIES, pnentries);
-                    break;
-                case 1: /* entry not sent */
-                    break;
-                case -1: /* connection closed */
-                    /*
-                     * mark the operation as abandoned so the backend
-                     * next entry function gets called again and has
-                     * a chance to clean things up.
-                     */
-                    operation->o_status = SLAPI_OP_STATUS_ABANDONED;
-                    break;
-                }
+    /*
+     * It's a regular entry, or it's a referral and
+     * managedsait control is on.  In either case, send
+     * the entry.
+     */
+    switch (send_ldap_search_entry(pb, e, NULL, attrs, attrsonly)) {
+    case 0: /* entry sent ok */
+        (*pnentries)++;
+        slapi_pblock_set(pb, SLAPI_NENTRIES, pnentries);
+        break;
+    case 1: /* entry not sent */
+        break;
+    case -1: /* connection closed */
+        /*
+         * mark the operation as abandoned so the backend
+         * next entry function gets called again and has
+         * a chance to clean things up.
+         */
+        operation->o_status = SLAPI_OP_STATUS_ABANDONED;
+        break;
+    }
 }
 
 /* Loops through search entries and sends them to the client.
@@ -1565,21 +1617,34 @@ op_shared_log_error_access(Slapi_PBlock *pb, const char *type, const char *dn, c
     char *proxystr = NULL;
     Slapi_Operation *operation;
     Connection *pb_conn;
+    int32_t log_format = config_get_accesslog_log_format();
+    slapd_log_pblock logpb = {0};
 
     slapi_pblock_get(pb, SLAPI_OPERATION, &operation);
     slapi_pblock_get(pb, SLAPI_CONNECTION, &pb_conn);
 
-    if ((proxyauth_get_dn(pb, &proxydn, NULL) == LDAP_SUCCESS)) {
+    if ((proxyauth_get_dn(pb, &proxydn, NULL) == LDAP_SUCCESS &&
+        log_format == LOG_FORMAT_DEFAULT))
+    {
         proxystr = slapi_ch_smprintf(" authzid=\"%s\"", proxydn);
     }
 
-    slapi_log_access(LDAP_DEBUG_STATS, "conn=%" PRIu64 " op=%d %s dn=\"%s\"%s, %s\n",
-                     (pb_conn ? pb_conn->c_connid : 0),
-                     (operation ? operation->o_opid : 0),
-                     type,
-                     dn,
-                     proxystr ? proxystr : "",
-                     msg ? msg : "");
+    if (log_format != LOG_FORMAT_DEFAULT) {
+        slapd_log_pblock_init(&logpb, log_format, pb);
+        logpb.authzid = proxydn;
+        logpb.target_dn = dn;
+        logpb.msg = msg;
+        logpb.op_type = type;
+        slapd_log_access_error(&logpb);
+    } else {
+        slapi_log_access(LDAP_DEBUG_STATS, "conn=%" PRIu64 " op=%d %s dn=\"%s\"%s, %s\n",
+                         pb_conn ? pb_conn->c_connid : 0,
+                         operation ? operation->o_opid : 0,
+                         type,
+                         dn,
+                         proxystr ? proxystr : "",
+                         msg ? msg : "");
+    }
 
     slapi_ch_free_string(&proxydn);
     slapi_ch_free_string(&proxystr);
