@@ -4,22 +4,31 @@ import {
     expandable
 } from '@patternfly/react-table';
 import {
-    CatalogIcon,
-    ExclamationTriangleIcon,
-    ResourcesEmptyIcon,
     AngleRightIcon,
+    CatalogIcon,
+    ExclamationCircleIcon,
+    ExclamationTriangleIcon,
     InfoCircleIcon,
+    LockIcon,
+    ResourcesEmptyIcon,
 } from '@patternfly/react-icons';
 import {
     Breadcrumb,
     BreadcrumbItem,
+    Button,
+    Icon ,
+    Modal,
+    ModalVariant,
     Spinner,
     Tab,
     Tabs,
     TabTitleText,
     Text,
     TextContent,
+    TextList,
+    TextListItem,
     TextVariants,
+    Tooltip,
 } from "@patternfly/react-core";
 import {
     generateUniqueId,
@@ -35,23 +44,22 @@ import EditorTableView from './lib/ldap_editor/tableView.jsx';
 import EditorTreeView from './lib/ldap_editor/treeView.jsx';
 import { SearchDatabase } from './lib/ldap_editor/search.jsx';
 import GenericWizard from './lib/ldap_editor/wizards/genericWizard.jsx';
-import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
-import {
-    faSyncAlt
-} from '@fortawesome/free-solid-svg-icons';
-import '@fortawesome/fontawesome-svg-core/styles.css';
+import { SyncAltIcon } from '@patternfly/react-icons';
 import { log_cmd } from "./lib/tools.jsx";
 
+const _ = cockpit.gettext;
+
 export class LDAPEditor extends React.Component {
     constructor (props) {
         super(props);
 
         this.rootSuffixesRows = [];
         this.rootSuffixesTreeData = [];
-        this.initialChildText = 'Loading...';
+        this.initialChildText = _("Loading...");
 
         this.state = {
             activeTabKey: 0,
+            firstLoad: true,
             keyIndex: 0,
             suffixList: [],
             changeLayout: false,
@@ -79,24 +87,33 @@ export class LDAPEditor extends React.Component {
             refreshEntryTime: 0,
             navItems: [
                 { id: 'home', label: <CatalogIcon />, active: false },
-                { id: 'db-suffixes', to: '#', label: 'Database Suffixes', active: true }
+                { id: 'db-suffixes', to: '#', label: _("Database Suffixes"), active: true }
             ],
             columns: [
                 {
-                    title: 'Database Suffixes',
+                    title: _("Database Suffixes"),
                     cellFormatters: [expandable]
                 },
                 {
-                    title: 'Child Entries'
+                    title: _("Child Entries")
                 },
                 {
-                    title: 'Last Modified'
+                    title: _("Last Modified")
                 }
             ],
             rows: [],
             pagedRows: [],
             refreshing: false,
             allObjectclasses: [],
+            isConfirmModalOpen: false,
+            isTreeViewAction: false,
+            currentRowKey: -1
+        };
+
+        this.handleConfirmModalToggle = () => {
+            this.setState(({ isConfirmModalOpen }) => ({
+                isConfirmModalOpen: !isConfirmModalOpen,
+            }));
         };
 
         // Toggle currently active tab
@@ -107,7 +124,7 @@ export class LDAPEditor extends React.Component {
         };
 
         // Actions when user clicks on the Entry Details menu
-        this.handleToggleEntryMenu = isOpen => {
+        this.onToggleEntryMenu = (_event, isOpen) => {
             this.setState({
                 entryMenuIsOpen: isOpen
             });
@@ -132,6 +149,42 @@ export class LDAPEditor extends React.Component {
                 });
                 return;
             }
+            if (aTarget.name === ENTRY_MENU.lockRole) {
+                this.setState({
+                    entryDn: aTarget.value,
+                    entryType: "role",
+                    operationType: "lock",
+                    isTreeViewAction: true
+                }, () => { this.handleConfirmModalToggle() });
+                return;
+            }
+            if (aTarget.name === ENTRY_MENU.lockAccount) {
+                this.setState({
+                    entryDn: aTarget.value,
+                    entryType: "account",
+                    operationType: "lock",
+                    isTreeViewAction: true
+                }, () => { this.handleConfirmModalToggle() });
+                return;
+            }
+            if (aTarget.name === ENTRY_MENU.unlockRole) {
+                this.setState({
+                    entryDn: aTarget.value,
+                    entryType: "role",
+                    operationType: "unlock",
+                    isTreeViewAction: true
+                }, () => { this.handleConfirmModalToggle() });
+                return;
+            }
+            if (aTarget.name === ENTRY_MENU.unlockAccount) {
+                this.setState({
+                    entryDn: aTarget.value,
+                    entryType: "account",
+                    operationType: "unlock",
+                    isTreeViewAction: true
+                }, () => { this.handleConfirmModalToggle() });
+                return;
+            }
 
             const keyIndex = this.state.keyIndex + 1;
             this.setState({
@@ -187,6 +240,7 @@ export class LDAPEditor extends React.Component {
         this.onNavItemClick = this.onNavItemClick.bind(this);
         this.handleReload = this.handleReload.bind(this);
         this.getAttributes = this.getAttributes.bind(this);
+        this.handleLockUnlockEntry = this.handleLockUnlockEntry.bind(this);
     }
 
     handleReload(refresh) {
@@ -196,6 +250,12 @@ export class LDAPEditor extends React.Component {
             addNotification: this.props.addNotification,
         };
 
+        if (this.state.firstLoad) {
+            this.setState({
+                firstLoad: false
+            });
+        }
+
         this.setState({
             searching: true,
             loading: refresh
@@ -210,17 +270,73 @@ export class LDAPEditor extends React.Component {
                 const params = {
                     serverId: this.props.serverId,
                     baseDn: suffixDN,
-                    parentId: parentId,
+                    parentId,
                     addNotification: this.props.addNotification,
                 };
 
                 getRootSuffixEntryDetails(params, this.updateTableRootSuffixes);
                 parentId += 2; // The next DN row will be two rows below.
+                return [];
             });
             getOneLevelEntries(params, this.processDirectChildren);
         }
     }
 
+    handleLockUnlockEntry() {
+        const {
+            entryDn,
+            entryType,
+            operationType,
+            isTreeViewAction
+        } = this.state;
+
+        const cmd = ["dsidm", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
+            "-b", entryDn, entryType, operationType, entryDn];
+        log_cmd("handleLockUnlockEntry", `${operationType} entry`, cmd);
+        cockpit
+                .spawn(cmd, { superuser: true, err: 'message' })
+                .done(_ => {
+                    this.setState({
+                        entryMenuIsOpen: !this.state.entryMenuIsOpen,
+                    }, () => {
+                        this.handleConfirmModalToggle();
+                        if (isTreeViewAction) {
+                            this.setState({
+                                refreshEntryTime: Date.now(),
+                                isTreeViewAction: false
+                            });
+                        } else {
+                            this.handleReload(true);
+                        }
+                    });
+                })
+                .fail(err => {
+                    const errMsg = JSON.parse(err);
+                    console.error(
+                        "handleLockUnlockEntry",
+                        `${entryType} ${operationType} operation failed -`,
+                        errMsg.desc
+                    );
+                    this.props.addNotification(
+                        `${errMsg.desc.includes(`is already ${operationType === "unlock" ? "active" : "locked"}`) ? 'warning' : 'error'}`,
+                        `${errMsg.desc}`
+                    );
+                    this.setState({
+                        entryMenuIsOpen: !this.state.entryMenuIsOpen,
+                    }, () => {
+                        this.handleConfirmModalToggle();
+                        if (isTreeViewAction) {
+                            this.setState({
+                                refreshEntryTime: Date.now(),
+                                isTreeViewAction: false
+                            });
+                        } else {
+                            this.handleReload(true);
+                        }
+                    });
+                });
+    }
+
     getAttributes(callbackFunc) {
         const attr_cmd = [
             "dsconf",
@@ -254,6 +370,18 @@ export class LDAPEditor extends React.Component {
         });
     }
 
+    componentDidUpdate(prevProps) {
+        if (this.props.wasActiveList.includes(7)) {
+            if (this.state.firstLoad) {
+                this.handleReload(true);
+            } else {
+                if (this.props.serverId !== prevProps.serverId) {
+                    this.handleReload(true);
+                }
+            }
+        }
+    }
+
     getPageData (page, perPage) {
         if (page === 1) {
             const pagedRows = this.state.rows.slice(0, 2 * perPage); // Each parent has a single child.
@@ -312,7 +440,7 @@ export class LDAPEditor extends React.Component {
             };
             getOneLevelEntries(params, this.processDirectChildren);
         }
-    }
+    };
 
     // Handle clicks on the numSubordinates links.
     handleClickNumSubordinates = (dn) => {
@@ -354,7 +482,7 @@ export class LDAPEditor extends React.Component {
             this.setState(prevState => ({
                 columns: [
                     {
-                        title: 'Entry DN',
+                        title: _("Entry DN"),
                         cellFormatters: [expandable]
                     },
                     ...prevState.columns.slice(1)
@@ -372,7 +500,7 @@ export class LDAPEditor extends React.Component {
             addNotification: this.props.addNotification,
         };
         getOneLevelEntries(params, this.processDirectChildren);
-    }
+    };
 
     // Process the entries that are direct children.
     processDirectChildren = (directChildren, params) => {
@@ -395,12 +523,15 @@ export class LDAPEditor extends React.Component {
 
             // TODO Test for a JPEG photo!!!
             // if ( info.fullEntry.contains)
+
+            // TODO Add isActive func
             let dn = info.dn;
             if (info.ldapsubentry) {
-                dn =
+                dn = (
                     <div className="ds-info-icon">
-                        {info.dn} <InfoCircleIcon title="This is a hidden LDAP subentry" className="ds-info-icon" />
-                    </div>;
+                        {info.dn} <InfoCircleIcon title={_("This is a hidden LDAP subentry")} className="ds-info-icon" />
+                    </div>
+                );
             }
 
             childrenRows.push(
@@ -411,7 +542,11 @@ export class LDAPEditor extends React.Component {
                         numSubCellInfo,
                         info.modifyTimestamp,
                     ],
-                    rawdn: info.dn
+                    rawdn: info.dn,
+                    entryState: "",
+                    isRole: info.isRole,
+                    isLockable: info.isLockable,
+                    ldapsubentry: info.ldapsubentry
                 },
                 {
                     // customRowId: info.parentId + 1,
@@ -423,6 +558,7 @@ export class LDAPEditor extends React.Component {
 
             // Increment by 2 the row number.
             rowNumber += 2;
+            return [];
         });
 
         this.setState({
@@ -433,8 +569,12 @@ export class LDAPEditor extends React.Component {
             pagedRows: childrenRows.slice(0, 2 * this.state.perPage),
             total: childrenRows.length / 2,
             page: 1
+        }, () => {
+            if (this.state.currentRowKey >= 0) {
+                this.handleCollapse(null, this.state.currentRowKey, true, null);
+            }
         });
-    }
+    };
 
     handleCollapse (event, rowKey, isOpen, data) {
         const { pagedRows } = this.state;
@@ -452,24 +592,117 @@ export class LDAPEditor extends React.Component {
 
         const firstTime = (pagedRows[rowKey + 1].cells[0].title) === this.initialChildText;
         if (firstTime) {
-            const baseDn = pagedRows[rowKey].rawdn; // The DN is the first element in the array.
-            getBaseLevelEntryAttributes(this.props.serverId, baseDn, (entryArray) => {
-                pagedRows[rowKey + 1].cells = [{
-                    title: (
-                        <>
-                            {entryArray.map((line) => (
-                                <div key={line.attribute + line.value}>
-                                    <strong>{line.attribute}</strong>
-                                    {line.value.toLowerCase() === ": ldapsubentry" ? <span className="ds-info-color">{line.value}</span> : line.attribute.toLowerCase() === "userpassword" ? ": ********" : line.value}
-                                </div>
-                            ))}
-                        </>
-                    )
-                }];
-                // Update the row.
-                this.setState({
-                    pagedRows
+            const entryRows = [];
+            let entryStateIcon = "";
+            let isRole = false;
+            const entryDn = pagedRows[rowKey].rawdn; // The DN is the first element in the array.
+            getBaseLevelEntryAttributes(this.props.serverId, entryDn, (entryArray) => {
+                entryArray.map(line => {
+                    const attr = line.attribute;
+                    const attrJpegVal = line.attribute.toLowerCase() === "jpegphoto"
+                        ? (
+                            <div>
+                                <img
+                                    src={`data:image/png;base64,${line.value.substr(3)}`} // strip ':: '
+                                    alt=''
+                                    style={{ width: '256px' }} // height will adjust automatically.
+                                />
+                            </div>
+                        )
+                        : line.value;
+                    const val = line.value.toLowerCase() === ": ldapsubentry" ? <span className="ds-info-color">{line.value}</span> : line.attribute.toLowerCase() === "userpassword" ? ": ********" : attrJpegVal;
+
+                    // <div key={line.attribute + line.value}></div>
+                    entryRows.push({ attr, value: val });
+
+                    const myVal = line.value.substring(1).trim()
+                            .toLowerCase();
+                    const accountObjectclasses = ['nsaccount', 'nsperson', 'simplesecurityobject',
+                        'organization', 'person', 'account', 'organizationalunit',
+                        'netscapeserver', 'domain', 'posixaccount', 'shadowaccount',
+                        'posixgroup', 'mailrecipient', 'nsroledefinition'];
+                    if (accountObjectclasses.includes(myVal)) {
+                        entryStateIcon = <LockIcon className="ds-pf-blue-color" />;
+                    }
+                    if (myVal === 'nsroledefinition') {
+                        isRole = true;
+                    }
+                    return [];
                 });
+                let entryState = "";
+                const cmd = ["dsidm", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
+                    "-b", entryDn, (isRole ? "role" : "account"), "entry-status", entryDn];
+                log_cmd("handleCollapse", "Checking if entry is activated", cmd);
+                cockpit
+                        .spawn(cmd, { superuser: true, err: 'message' })
+                        .done(content => {
+                            if ((entryDn !== 'Root DSE') && (entryStateIcon !== "")) {
+                                const status = JSON.parse(content);
+                                entryState = status.info.state;
+                                if (entryState === 'inactivity limit exceeded' || entryState.startsWith("probably activated or")) {
+                                    entryStateIcon = <ExclamationTriangleIcon className="ds-pf-yellow-color ct-icon-exclamation-triangle" />;
+                                }
+                            }
+                        })
+                        .fail(err => {
+                            const errMsg = JSON.parse(err);
+                            if ((entryDn !== 'Root DSE') && (entryStateIcon !== "") && !(errMsg.desc.includes("Root suffix can't be locked or unlocked"))) {
+                                console.error(
+                                    "handleCollapse",
+                                    `${isRole ? "role" : "account"} account entry-status operation failed`,
+                                    errMsg.desc
+                                );
+                                entryState = _("error: please, check browser logs");
+                                entryStateIcon = <ExclamationCircleIcon className="ds-pf-red-color ct-exclamation-circle" />;
+                            }
+                        })
+                        .finally(() => {
+                            let entryStateIconFinal = "";
+                            if ((entryState !== "") && (entryStateIcon !== "") && (entryState !== "activated")) {
+                                entryStateIconFinal = (
+                                    <Tooltip
+                                        position="bottom"
+                                        content={
+                                            <div className="ds-info-icon">
+                                                {entryState}
+                                            </div>
+                                        }
+                                    >
+                                        <a className="ds-font-size-md">{entryStateIcon}</a>
+                                    </Tooltip>
+                                );
+                            }
+                            pagedRows[rowKey].entryState = entryState;
+                            pagedRows[rowKey + 1].cells = [{
+                                title: (
+                                    <>
+                                        {entryRows.map((line) => {
+                                            let attrLine = "";
+                                            if (line.attr === "dn") {
+                                                attrLine = (
+                                                    <div key={line.attr + line.value}>
+                                                        <strong>{line.attr}</strong>{line.value} {entryStateIconFinal}
+                                                    </div>
+                                                );
+                                            } else {
+                                                attrLine = (
+                                                    <div key={line.attr + line.value}>
+                                                        <strong>{line.attr}</strong>{line.value}
+                                                    </div>
+                                                );
+                                            }
+                                            return attrLine;
+                                        }
+                                        )}
+                                    </>
+                                )
+                            }];
+                            // Update the rows.
+                            this.setState({
+                                pagedRows,
+                                currentRowKey: -1
+                            });
+                        });
             });
         }
     }
@@ -521,7 +754,8 @@ export class LDAPEditor extends React.Component {
                     ],
                     rawdn: info.dn,
                     customRowId: info.parentId,
-                    isEmptySuffix: isEmptySuffix
+                    isEmptySuffix,
+                    entryState: ""
                 },
                 {
                     customRowId: info.parentId + 1,
@@ -540,9 +774,9 @@ export class LDAPEditor extends React.Component {
                             )
                             : (
                                 <div>
-                                    <strong>&nbsp;<em>This suffix is empty!&nbsp;&nbsp;&nbsp;</em></strong>
+                                    <strong>&nbsp;<em>{_("This suffix is empty!")}&nbsp;&nbsp;&nbsp;</em></strong>
                                     <a key={'top-entry-' + info.dn} href="#" onClick={() => { this.onHandleEmptySuffixToggle(info.dn) }}>
-                                        Click here to create the top entry
+                                        {_("Click here to create the top entry")}
                                     </a>
                                 </div>
                             )
@@ -556,7 +790,7 @@ export class LDAPEditor extends React.Component {
         const randomId = generateUniqueId();
         // const nodeChildren = [];
         const nodeChildren = [{
-            name: 'Loading...',
+            name: _("Loading..."),
             id: randomId,
             icon: <Spinner size="sm" />,
             isFakeEntry: true
@@ -571,7 +805,7 @@ export class LDAPEditor extends React.Component {
             // The DN value can also be retrieved from info.fullEntry
             // ( but will require to retrieve the DN line and then split it. Taking a lazy approach here ;-))
             dn: info.dn,
-            isEmptySuffix: isEmptySuffix,
+            isEmptySuffix,
             loadChildren: numSubValue > 0 // Used to insert child nodes when needed.
         };
 
@@ -626,7 +860,7 @@ export class LDAPEditor extends React.Component {
         } // TODO: If there was an error, the loading state would continue for ever.
         // Use better logic or render an empty table after a while ( 15 seconds? )
         // with some meaningful error message.
-    }
+    };
 
     // Process the root suffixes
     processRootSuffixes = (userSuffData) => {
@@ -652,14 +886,15 @@ export class LDAPEditor extends React.Component {
                 const params = {
                     serverId: this.props.serverId,
                     baseDn: suffixDN,
-                    parentId: parentId,
+                    parentId,
                     addNotification: this.props.addNotification,
                 };
                 getRootSuffixEntryDetails(params, this.updateTableRootSuffixes);
                 parentId += 2; // The next DN row will be two rows below.
+                return [];
             });
         });
-    }
+    };
 
     showSuffixes = (label, event) => {
         getUserSuffixes(this.props.serverId, this.processRootSuffixes);
@@ -676,7 +911,7 @@ export class LDAPEditor extends React.Component {
         // TODO ==> To optimize!
         if ((isRootSuff && (tableTitle !== 'Database Suffixes')) ||
             (!isRootSuff && (tableTitle === 'Database Suffixes'))) {
-            const firstTitle = (label === 'Database Suffixes') ? 'Database Suffixes' : 'Entry DN';
+            const firstTitle = (label === 'Database Suffixes') ? _("Database Suffixes") : _("Entry DN");
             this.setState(prevState => ({
                 columns: [
                     {
@@ -691,7 +926,7 @@ export class LDAPEditor extends React.Component {
                 searching: false
             });
         }
-    }
+    };
 
     actionResolver = (rowData, { rowIndex }) => {
         // No action on the children.
@@ -702,16 +937,65 @@ export class LDAPEditor extends React.Component {
         // If there is an entry suffix, allow only the creation of the top entry:
         if (rowData.isEmptySuffix) {
             return [{
-                title: 'Create Top Entry...',
+                title: _("Create Top Entry..."),
                 onClick: () => {
                     this.onHandleEmptySuffixToggle(rowData.rawdn);
                 }
             }];
         }
+
+        let lockingDropdown = [];
+        if (rowData.isLockable) {
+            if (rowData.entryState !== "" && rowData.entryState !== "activated") {
+                if (rowData.entryState.includes("probably activated") || rowData.entryState.includes("indirectly locked")) {
+                    lockingDropdown = [{
+                        title: _("Lock ..."),
+                        onClick:
+                        () => {
+                            const entryType = rowData.isRole ? "role" : "account";
+                            this.setState({
+                                entryDn: rowData.rawdn,
+                                entryType,
+                                operationType: "lock",
+                                currentRowKey: rowData.secretTableRowKeyId
+                            }, () => { this.handleConfirmModalToggle() });
+                        }
+                    }];
+                } else {
+                    lockingDropdown = [{
+                        title: _("Unlock ..."),
+                        onClick:
+                        () => {
+                            const entryType = rowData.isRole ? "role" : "account";
+                            this.setState({
+                                entryDn: rowData.rawdn,
+                                entryType,
+                                operationType: "unlock",
+                                currentRowKey: rowData.secretTableRowKeyId
+                            }, () => { this.handleConfirmModalToggle() });
+                        }
+                    }];
+                }
+            } else if (rowData.entryState === "activated") {
+                lockingDropdown = [{
+                    title: _("Lock ..."),
+                    onClick:
+                    () => {
+                        const entryType = rowData.isRole ? "role" : "account";
+                        this.setState({
+                            entryDn: rowData.rawdn,
+                            entryType,
+                            operationType: "lock",
+                            currentRowKey: rowData.secretTableRowKeyId
+                        }, () => { this.handleConfirmModalToggle() });
+                    }
+                }];
+            }
+        }
         const keyIndex = this.state.keyIndex + 1;
         const updateActions =
             [{
-                title: 'Search ...',
+                title: _("Search ..."),
                 onClick:
                 () => {
                     this.setState({
@@ -724,7 +1008,7 @@ export class LDAPEditor extends React.Component {
                 isSeparator: true
             },
             {
-                title: 'Edit ...',
+                title: _("Edit ..."),
                 onClick:
                 () => {
                     this.setState({
@@ -736,7 +1020,7 @@ export class LDAPEditor extends React.Component {
                 }
             },
             {
-                title: 'New ...',
+                title: _("New ..."),
                 onClick:
                 () => {
                     this.setState({
@@ -748,7 +1032,7 @@ export class LDAPEditor extends React.Component {
                 }
             },
             {
-                title: 'Rename ...',
+                title: _("Rename ..."),
                 isDisabled: rowData.customRowId === 0, // can not rename root suffix
                 onClick:
                 () => {
@@ -760,8 +1044,9 @@ export class LDAPEditor extends React.Component {
                     });
                 }
             },
+            ...lockingDropdown,
             {
-                title: 'ACIs ...',
+                title: _("ACIs ..."),
                 onClick:
                 () => {
                     this.setState({
@@ -774,7 +1059,7 @@ export class LDAPEditor extends React.Component {
                 }
             },
             {
-                title: 'Class of Service ...',
+                title: _("Class of Service ..."),
                 onClick:
                 () => {
                     this.setState({
@@ -786,26 +1071,11 @@ export class LDAPEditor extends React.Component {
                     });
                 }
             },
-            /*
-            {
-                title: 'Roles ...',
-                isDisabled: true,
-                onClick: (event, rowId, rowData, extra) => {
-                    // TODO
-                    console.log(`clicked on Third action, on row ${rowId}`);
-                    console.log('extra = ' + extra);
-                }
-            },
-            {
-                title: 'Smart Referrals ...',
-                isDisabled: true
-            },
-            */
             {
                 isSeparator: true
             },
             {
-                title: 'Delete ...',
+                title: _("Delete ..."),
                 onClick:
                 () => {
                     this.setState({
@@ -820,7 +1090,7 @@ export class LDAPEditor extends React.Component {
                 isSeparator: true
             },
             {
-                title: 'Refresh ...',
+                title: _("Refresh ..."),
                 onClick:
                 () => {
                     this.handleReload(true);
@@ -830,7 +1100,7 @@ export class LDAPEditor extends React.Component {
         return [
             ...updateActions,
         ];
-    }
+    };
 
     render () {
         const {
@@ -879,6 +1149,7 @@ export class LDAPEditor extends React.Component {
                         treeViewRootSuffixes={this.state.treeViewRootSuffixes}
                         setWizardOperationInfo={this.setWizardOperationInfo}
                         onReload={this.handleReloadNoop}
+                        // coverity[copy_paste_error]
                         onModrdnReload={this.handleReload}
                         allObjectclasses={this.state.allObjectclasses}
                         addNotification={this.props.addNotification}
@@ -886,10 +1157,10 @@ export class LDAPEditor extends React.Component {
                     />
                 )}
                 <Tabs isBox className="ds-margin-top-lg ds-indent" activeKey={this.state.activeTabKey} onSelect={this.handleNavSelect}>
-                    <Tab eventKey={0} title={<TabTitleText>Tree View</TabTitleText>}>
+                    <Tab eventKey={0} title={<TabTitleText>{_("Tree View")}</TabTitleText>}>
                         <EditorTreeView
                             key={loading}
-                            onToggleEntryMenu={this.handleToggleEntryMenu}
+                            handleToggleEntryMenu={this.onToggleEntryMenu}
                             onSelectEntryOptions={this.handleSelectEntryOptions}
                             entryMenuIsOpen={entryMenuIsOpen}
                             treeViewRootSuffixes={treeViewRootSuffixes}
@@ -905,7 +1176,7 @@ export class LDAPEditor extends React.Component {
                             addNotification={this.props.addNotification}
                         />
                     </Tab>
-                    <Tab eventKey={1} title={<TabTitleText>Table View</TabTitleText>}>
+                    <Tab eventKey={1} title={<TabTitleText>{_("Table View")}</TabTitleText>}>
                         <div className={this.state.searching ? "ds-disabled" : ""}>
                             <Breadcrumb className="ds-left-margin ds-margin-top-xlg">
                                 {navItems.map(({ id, to, label, active }) => (
@@ -914,17 +1185,17 @@ export class LDAPEditor extends React.Component {
                                     </BreadcrumbItem>
                                 ))}
                             </Breadcrumb>
-                            <FontAwesomeIcon
-                                className="ds-left-margin ds-refresh"
-                                icon={faSyncAlt}
-                                title="Refresh"
-                                onClick={this.handleReload}
-                            />
+                            <Icon>
+                                <SyncAltIcon
+                                    className="ds-left-margin ds-refresh"
+                                    onClick={this.handleReload}
+                                />
+                            </Icon>
                         </div>
                         <div className={this.state.searching ? "ds-margin-top-xlg ds-center" : "ds-hidden"}>
                             <TextContent>
                                 <Text component={TextVariants.h3}>
-                                    Loading <i>{this.state.searchBase}</i> ...
+                                    {_("Loading")} <i>{this.state.searchBase}</i> ...
                                 </Text>
                             </TextContent>
                             <Spinner className="ds-margin-top-lg" size="xl" />
@@ -948,7 +1219,7 @@ export class LDAPEditor extends React.Component {
                             />
                         </div>
                     </Tab>
-                    <Tab eventKey={2} title={<TabTitleText>Search</TabTitleText>}>
+                    <Tab eventKey={2} title={<TabTitleText>{_("Search")}</TabTitleText>}>
                         <SearchDatabase
                             key={this.state.suffixList + this.state.searchBase}
                             serverId={this.props.serverId}
@@ -960,13 +1231,65 @@ export class LDAPEditor extends React.Component {
                         />
                     </Tab>
                 </Tabs>
-                { this.state.showEmptySuffixModal &&
+                {this.state.showEmptySuffixModal &&
                     <CreateRootSuffix
                         showEmptySuffixModal={this.state.showEmptySuffixModal}
                         handleEmptySuffixToggle={this.onHandleEmptySuffixToggle}
                         suffixDn={this.state.emptyDN}
                         editorLdapServer={this.props.serverId}
                     />}
+                <Modal
+                    // TODO: Fix confirmation modal formatting and size; add operation to the tables
+                    variant={ModalVariant.medium}
+                    title={
+                        cockpit.format(_("Are you sure you want to $0 the $1?"), this.state.operationType, this.state.entryType)
+                    }
+                    isOpen={this.state.isConfirmModalOpen}
+                    onClose={this.handleConfirmModalToggle}
+                    actions={[
+                        <Button key="confirm" variant="primary" onClick={this.handleLockUnlockEntry}>
+                            {_("Confirm")}
+                        </Button>,
+                        <Button key="cancel" variant="link" onClick={this.handleConfirmModalToggle}>
+                            {_("Cancel")}
+                        </Button>
+                    ]}
+                >
+                    <TextContent className="ds-margin-top ds-hide-vertical-scrollbar">
+                        <Text>
+                            {this.state.entryType === "account" && (() => {
+                                const value1 = this.state.operationType === "lock" ? _("add") : _("remove");
+                                const value2 = this.state.operationType === "lock" ? _("to") : _("from");
+
+                                return cockpit.format(_("It will $0 nsAccountLock attribute $1 the entry - $2."), value1, value2, this.state.entryDn);
+                            })()}
+                            {!this.state.entryType === "account" && _("This operation will make sure that these five entries are created at the entry's root suffix (if not, they will be created):")}
+                        </Text>
+                        {this.state.entryType === "role" &&
+                        <>
+                            <TextList>
+                                <TextListItem>
+                                    cn=nsManagedDisabledRole
+                                </TextListItem>
+                                <TextListItem>
+                                    cn=nsDisabledRole
+                                </TextListItem>
+                                <TextListItem>
+                                    cn=nsAccountInactivationTmp (with a child)
+                                </TextListItem>
+                                <TextListItem>
+                                    cn=nsAccountInactivation_cos
+                                </TextListItem>
+                            </TextList>
+                            <Text>
+                                {(() => {
+                                    const value1 = this.state.operationType === "lock" ? _("added to") : _("removed from");
+                                    return cockpit.format(_("The entry - $0 - will be $1 nsRoleDN attribute in cn=nsDisabledRole entry in the root suffix."), this.state.entryDn, value1);
+                                })()}
+                            </Text>
+                        </>}
+                    </TextContent>
+                </Modal>
             </>
         );
     }
