@@ -7,13 +7,6 @@ import { SuffixReferrals } from "./referrals.jsx";
 import { SuffixIndexes } from "./indexes.jsx";
 import { VLVIndexes } from "./vlvIndexes.jsx";
 import { log_cmd, bad_file_name } from "../tools.jsx";
-import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
-import {
-    faLeaf,
-    faTree,
-    faSyncAlt
-} from '@fortawesome/free-solid-svg-icons';
-import '@fortawesome/fontawesome-svg-core/styles.css';
 import {
     ImportModal,
     ExportModal,
@@ -21,20 +14,30 @@ import {
     CreateLinkModal,
 } from "./databaseModal.jsx";
 import {
-    Dropdown,
-    DropdownToggle,
-    DropdownItem,
-    DropdownPosition,
-    DropdownSeparator,
-    Grid,
-    GridItem,
-    Tab,
-    Tabs,
-    TabTitleText,
-} from "@patternfly/react-core";
+    Button,
+	Grid,
+	GridItem,
+	Tab,
+	Tabs,
+	TabTitleText
+} from '@patternfly/react-core';
+import {
+    FolderIcon,
+    LeafIcon,
+    SyncAltIcon
+} from '@patternfly/react-icons';
+import {
+	Dropdown,
+	DropdownToggle,
+	DropdownItem,
+	DropdownPosition,
+	DropdownSeparator
+} from '@patternfly/react-core/deprecated';
 
 import PropTypes from "prop-types";
 
+const _ = cockpit.gettext;
+
 export class Suffix extends React.Component {
     constructor (props) {
         super(props);
@@ -108,12 +111,12 @@ export class Suffix extends React.Component {
         // config.autoAddCss = false;
 
         // Dropdown tasks
-        this.onToggle = dropdownIsOpen => {
+        this.handleToggle = (_event, dropdownIsOpen) => {
             this.setState({
                 dropdownIsOpen
             });
         };
-        this.onSelect = event => {
+        this.handleSelect = event => {
             this.setState({
                 dropdownIsOpen: !this.state.dropdownIsOpen
             });
@@ -132,39 +135,39 @@ export class Suffix extends React.Component {
 
         // General bindings
         // Import modal
-        this.showImportModal = this.showImportModal.bind(this);
+        this.handleShowImportModal = this.handleShowImportModal.bind(this);
         this.closeImportModal = this.closeImportModal.bind(this);
-        this.handleChange = this.handleChange.bind(this);
+        this.onChange = this.onChange.bind(this);
         this.doImport = this.doImport.bind(this);
         this.importLDIF = this.importLDIF.bind(this);
         this.showConfirmLDIFImport = this.showConfirmLDIFImport.bind(this);
         this.closeConfirmLDIFImport = this.closeConfirmLDIFImport.bind(this);
         // Export modal
-        this.showExportModal = this.showExportModal.bind(this);
+        this.handleShowExportModal = this.handleShowExportModal.bind(this);
         this.closeExportModal = this.closeExportModal.bind(this);
         this.doExport = this.doExport.bind(this);
-        this.handleExportChange = this.handleExportChange.bind(this);
+        this.onExportChange = this.onExportChange.bind(this);
         // Reindex Suffix Modal
-        this.showReindexConfirm = this.showReindexConfirm.bind(this);
+        this.handleShowReindexConfirm = this.handleShowReindexConfirm.bind(this);
         this.closeReindexConfirm = this.closeReindexConfirm.bind(this);
         this.doReindex = this.doReindex.bind(this);
         // Create sub suffix modal
-        this.showSubSuffixModal = this.showSubSuffixModal.bind(this);
+        this.handleShowSubSuffixModal = this.handleShowSubSuffixModal.bind(this);
         this.closeSubSuffixModal = this.closeSubSuffixModal.bind(this);
         this.createSubSuffix = this.createSubSuffix.bind(this);
-        this.handleSubSuffixOnSelect = this.handleSubSuffixOnSelect.bind(this);
-        this.handleSubSuffixChange = this.handleSubSuffixChange.bind(this);
+        this.onSubSuffixOnSelect = this.onSubSuffixOnSelect.bind(this);
+        this.onSubSuffixChange = this.onSubSuffixChange.bind(this);
         // Create link modal
-        this.showLinkModal = this.showLinkModal.bind(this);
+        this.handleShowLinkModal = this.handleShowLinkModal.bind(this);
         this.closeLinkModal = this.closeLinkModal.bind(this);
         this.createLink = this.createLink.bind(this);
-        this.handleLinkChange = this.handleLinkChange.bind(this);
-        this.handleLinkOnSelect = this.handleLinkOnSelect.bind(this);
+        this.onLinkChange = this.onLinkChange.bind(this);
+        this.onLinkOnSelect = this.onLinkOnSelect.bind(this);
         // Suffix config
         this.saveSuffixConfig = this.saveSuffixConfig.bind(this);
-        this.showDeleteConfirm = this.showDeleteConfirm.bind(this);
+        this.handleShowDeleteConfirm = this.handleShowDeleteConfirm.bind(this);
         this.closeDeleteConfirm = this.closeDeleteConfirm.bind(this);
-        this.handleConfigChange = this.handleConfigChange.bind(this);
+        this.onConfigChange = this.onConfigChange.bind(this);
         this.doDelete = this.doDelete.bind(this);
     }
 
@@ -175,7 +178,7 @@ export class Suffix extends React.Component {
     //
     // Import Modal
     //
-    showImportModal() {
+    handleShowImportModal() {
         this.setState({
             ldifLocation: "",
             attrEncryption: false,
@@ -190,21 +193,21 @@ export class Suffix extends React.Component {
         });
     }
 
-    handleChange(e) {
+    onChange(e) {
         const value = e.target.type === 'checkbox' ? e.target.checked : e.target.value;
         let valueErr = false;
         const errObj = this.state.errObj;
-        if (value == "") {
+        if (value === "") {
             valueErr = true;
         }
         errObj[e.target.id] = valueErr;
         this.setState({
             [e.target.id]: value,
-            errObj: errObj
+            errObj
         });
     }
 
-    handleConfigChange(e) {
+    onConfigChange(e) {
         const value = e.target.type === 'checkbox' ? e.target.checked : e.target.value;
         const attr = e.target.id;
         let saveBtnDisabled = true;
@@ -214,18 +217,18 @@ export class Suffix extends React.Component {
             'readOnly', 'requireIndex', 'dbstate'
         ];
         for (const check_attr of configAttrs) {
-            if (attr != check_attr) {
-                if (this.state[check_attr] != this.state['_' + check_attr]) {
+            if (attr !== check_attr) {
+                if (this.state[check_attr] !== this.state['_' + check_attr]) {
                     saveBtnDisabled = false;
                 }
-            } else if (value != this.state['_' + check_attr]) {
+            } else if (value !== this.state['_' + check_attr]) {
                 saveBtnDisabled = false;
             }
         }
 
         this.setState({
             [attr]: value,
-            saveBtnDisabled: saveBtnDisabled
+            saveBtnDisabled
         });
     }
 
@@ -265,7 +268,7 @@ export class Suffix extends React.Component {
                 .done(content => {
                     this.props.addNotification(
                         "success",
-                        `Import successfully initiated`
+                        _("Import successfully initiated")
                     );
                     this.setState({
                         modalSpinning: false,
@@ -277,7 +280,7 @@ export class Suffix extends React.Component {
                     const errMsg = JSON.parse(err);
                     this.props.addNotification(
                         "error",
-                        `Error importing LDIF file - ${errMsg.desc}`
+                        cockpit.format(_("Error importing LDIF file - $0"), errMsg.desc)
                     );
                     this.setState({
                         modalSpinning: false,
@@ -288,7 +291,7 @@ export class Suffix extends React.Component {
 
     doImport() {
         // Validate form before proceeding
-        if (this.state.ldifLocation != "") {
+        if (this.state.ldifLocation !== "") {
             this.setState({
                 showConfirmLDIFImport: true,
                 importLDIFName: this.state.ldifLocation,
@@ -299,7 +302,7 @@ export class Suffix extends React.Component {
     //
     // Export modal
     //
-    showExportModal() {
+    handleShowExportModal() {
         this.setState({
             ldifLocation: "",
             attrEncryption: false,
@@ -319,10 +322,10 @@ export class Suffix extends React.Component {
 
     doExport() {
         const missingArgs = { ldifLocation: false };
-        if (this.state.ldifLocation == "") {
+        if (this.state.ldifLocation === "") {
             this.props.addNotification(
                 "warning",
-                `LDIF name is empty`
+                _("LDIF name is empty")
             );
             missingArgs.ldifLocation = true;
             this.setState({
@@ -335,7 +338,7 @@ export class Suffix extends React.Component {
         if (bad_file_name(this.state.ldifLocation)) {
             this.props.addNotification(
                 "warning",
-                `LDIF name should not be a path.  All export files are stored in the server's LDIF directory`
+                _("LDIF name should not be a path.  All export files are stored in the server's LDIF directory")
             );
             missingArgs.ldifLocation = true;
             this.setState({
@@ -382,18 +385,18 @@ export class Suffix extends React.Component {
                                 const attrs = config.attrs;
                                 this.props.addNotification(
                                     "success",
-                                    `Database export complete. You can find the LDIF file in ${attrs['nsslapd-ldifdir'][0]} directory on the server machine.`
+                                    cockpit.format(_("Database export complete. You can find the LDIF file in $0 directory on the server machine."), attrs['nsslapd-ldifdir'][0])
                                 );
                             })
                             .fail(err => {
                                 const errMsg = JSON.parse(err);
                                 this.props.addNotification(
                                     "success",
-                                    `Database export complete.`
+                                    _("Database export complete.")
                                 );
                                 this.props.addNotification(
                                     "error",
-                                    `Error while trying to get the server's LDIF directory- ${errMsg.desc}`
+                                    cockpit.format(_("Error while trying to get the server's LDIF directory- $0"), errMsg.desc)
                                 );
                             });
                 })
@@ -402,7 +405,7 @@ export class Suffix extends React.Component {
                     this.props.reloadLDIFs();
                     this.props.addNotification(
                         "error",
-                        `Error exporting database - ${errMsg.desc}`
+                        cockpit.format(_("Error exporting database - $0"), errMsg.desc)
                     );
                     this.setState({
                         showExportModal: false,
@@ -413,7 +416,7 @@ export class Suffix extends React.Component {
     //
     // Reindex entire database
     //
-    showReindexConfirm() {
+    handleShowReindexConfirm() {
         this.setState({
             showReindexConfirm: true,
             modalChecked: false,
@@ -442,7 +445,7 @@ export class Suffix extends React.Component {
                 .done(content => {
                     this.props.addNotification(
                         "success",
-                        `Database has successfully been reindexed`
+                        _("Database has successfully been reindexed")
                     );
                     this.setState({
                         modalSpinning: false,
@@ -453,7 +456,7 @@ export class Suffix extends React.Component {
                     const errMsg = JSON.parse(err);
                     this.props.addNotification(
                         "error",
-                        `Failed to reindex database - ${errMsg.desc}`
+                        cockpit.format(_("Failed to reindex database - $0"), errMsg.desc)
                     );
                     this.setState({
                         modalSpinning: false,
@@ -465,7 +468,7 @@ export class Suffix extends React.Component {
     //
     // Create sub suffix
     //
-    showSubSuffixModal() {
+    handleShowSubSuffixModal() {
         this.setState({
             showSubSuffixModal: true,
             errObj: {},
@@ -506,7 +509,7 @@ export class Suffix extends React.Component {
                     this.closeSubSuffixModal();
                     this.props.addNotification(
                         "success",
-                        `Successfully created new sub-suffix`
+                        _("Successfully created new sub-suffix")
                     );
                     this.setState({
                         subSuffixSaving: false
@@ -518,7 +521,7 @@ export class Suffix extends React.Component {
                     this.closeSubSuffixModal();
                     this.props.addNotification(
                         "error",
-                        `Error creating sub-suffix - ${errMsg.desc}`
+                        cockpit.format(_("Error creating sub-suffix - $0"), errMsg.desc)
                     );
                     this.setState({
                         subSuffixSaving: false
@@ -529,7 +532,7 @@ export class Suffix extends React.Component {
     //
     // Create Chaining Link
     //
-    showLinkModal() {
+    handleShowLinkModal() {
         this.setState({
             showLinkModal: true,
             errObj: {},
@@ -568,7 +571,7 @@ export class Suffix extends React.Component {
                     this.closeLinkModal();
                     this.props.addNotification(
                         "success",
-                        `Successfully created database link`
+                        _("Successfully created database link")
                     );
                     this.setState({
                         linkSaving: false
@@ -580,7 +583,7 @@ export class Suffix extends React.Component {
                     this.closeLinkModal();
                     this.props.addNotification(
                         "error",
-                        `Error creating database link - ${errMsg.desc}`
+                        cockpit.format(_("Error creating database link - $0"), errMsg.desc)
                     );
                     this.setState({
                         linkSaving: false
@@ -588,13 +591,13 @@ export class Suffix extends React.Component {
                 });
     }
 
-    handleLinkOnSelect(value, event) {
+    onLinkOnSelect(value, event) {
         this.setState({
             createNsbindmechanism: value,
         });
     }
 
-    handleLinkChange(e) {
+    onLinkChange(e) {
         // Check for matching credentials
         const value = e.target.type === 'checkbox' ? e.target.checked : e.target.value;
         const attr = e.target.id;
@@ -608,19 +611,19 @@ export class Suffix extends React.Component {
             "createNsmultiplexorcredentialsConfirm", "createNsbindmechanism"
         ];
         for (const check_attr of check_attrs) {
-            if (attr != check_attr && this.state[check_attr] == "") {
+            if (attr !== check_attr && this.state[check_attr] === "") {
                 saveBtnDisabled = true;
             }
         }
 
         // Handle password validation
-        if (attr != "createNsmultiplexorcredentials" && attr != "createNsmultiplexorcredentialsConfirm") {
-            if (this.state.createNsmultiplexorcredentials != this.state.createNsmultiplexorcredentialsConfirm) {
+        if (attr !== "createNsmultiplexorcredentials" && attr !== "createNsmultiplexorcredentialsConfirm") {
+            if (this.state.createNsmultiplexorcredentials !== this.state.createNsmultiplexorcredentialsConfirm) {
                 saveBtnDisabled = true;
             }
         } else {
-            if (attr == "createNsmultiplexorcredentials") {
-                if (value != this.state.createNsmultiplexorcredentialsConfirm) {
+            if (attr === "createNsmultiplexorcredentials") {
+                if (value !== this.state.createNsmultiplexorcredentialsConfirm) {
                     errObj.createNsmultiplexorcredentials = true;
                     errObj.createNsmultiplexorcredentialsConfirm = true;
                     saveBtnDisabled = true;
@@ -628,8 +631,8 @@ export class Suffix extends React.Component {
                     errObj.createNsmultiplexorcredentials = false;
                     errObj.createNsmultiplexorcredentialsConfirm = false;
                 }
-            } else if (attr == "createNsmultiplexorcredentialsConfirm") {
-                if (value != this.state.createNsmultiplexorcredentials) {
+            } else if (attr === "createNsmultiplexorcredentialsConfirm") {
+                if (value !== this.state.createNsmultiplexorcredentials) {
                     errObj.createNsmultiplexorcredentials = true;
                     errObj.createNsmultiplexorcredentialsConfirm = true;
                     saveBtnDisabled = true;
@@ -640,26 +643,26 @@ export class Suffix extends React.Component {
             }
         }
 
-        if (value == "") {
+        if (value === "") {
             valueErr = true;
             saveBtnDisabled = true;
         }
         errObj[attr] = valueErr;
         this.setState({
             [e.target.id]: value,
-            errObj: errObj,
+            errObj,
             linkSaveBtnDisabled: saveBtnDisabled
         });
     }
 
-    handleSubSuffixOnSelect(value, event) {
+    onSubSuffixOnSelect(value, event) {
         let noInit = false;
         let addSuffix = false;
         let addSample = false;
 
-        if (value == "noInit") {
+        if (value === "noInit") {
             noInit = true;
-        } else if (value == "addSuffix") {
+        } else if (value === "addSuffix") {
             addSuffix = true;
         } else { // addSample
             addSample = true;
@@ -672,46 +675,46 @@ export class Suffix extends React.Component {
         });
     }
 
-    handleSubSuffixChange(e) {
+    onSubSuffixChange(e) {
         const value = e.target.value;
         let valueErr = false;
         const errObj = this.state.errObj;
         let saveBtnDisabled = false;
         const check_attrs = ["subSuffixBeName", "subSuffixValue"];
         for (const check_attr of check_attrs) {
-            if (this.state[check_attr] == "") {
+            if (this.state[check_attr] === "") {
                 saveBtnDisabled = true;
             }
         }
-        if (value == "") {
+        if (value === "") {
             valueErr = true;
             saveBtnDisabled = true;
         }
         errObj[e.target.id] = valueErr;
         this.setState({
             [e.target.id]: value,
-            errObj: errObj,
+            errObj,
             saveSubSuffixBtnDisabled: saveBtnDisabled
         });
     }
 
-    handleExportChange(e) {
+    onExportChange(e) {
         const value = e.target.type === 'checkbox' ? e.target.checked : e.target.value;
         let valueErr = false;
         const errObj = this.state.errObj;
         let saveBtnDisabled = false;
 
-        if (e.target.id != "ldifLocation" && this.state.ldifLocation == 0) {
+        if (e.target.id !== "ldifLocation" && this.state.ldifLocation === 0) {
             saveBtnDisabled = true;
         }
-        if (value == "") {
+        if (value === "") {
             valueErr = true;
             saveBtnDisabled = true;
         }
         errObj[e.target.id] = valueErr;
         this.setState({
             [e.target.id]: value,
-            errObj: errObj,
+            errObj,
             saveExportBtnDisabled: saveBtnDisabled
         });
     }
@@ -719,7 +722,7 @@ export class Suffix extends React.Component {
     //
     // Delete suffix
     //
-    showDeleteConfirm(item) {
+    handleShowDeleteConfirm(item) {
         this.setState({
             showDeleteConfirm: true,
             modalSpinning: false,
@@ -739,10 +742,10 @@ export class Suffix extends React.Component {
         // Delete suffix
         this.setState({
             modalSpinning: true
-        })
+        });
         const cmd = [
             "dsconf", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
-            "backend", "delete", this.props.suffix
+            "backend", "delete", this.props.suffix, "--do-it"
         ];
         log_cmd("doDelete", "Delete database", cmd);
         cockpit
@@ -752,7 +755,7 @@ export class Suffix extends React.Component {
                     this.closeDeleteConfirm();
                     this.props.addNotification(
                         "success",
-                        `Successfully deleted database`
+                        _("Successfully deleted database")
                     );
                 })
                 .fail(err => {
@@ -761,7 +764,7 @@ export class Suffix extends React.Component {
                     this.closeDeleteConfirm();
                     this.props.addNotification(
                         "error",
-                        `Error deleting database - ${errMsg.desc}`
+                        cockpit.format(_("Error deleting database - $0"), errMsg.desc)
                     );
                 });
     }
@@ -774,33 +777,33 @@ export class Suffix extends React.Component {
             'backend', 'suffix', 'set', this.props.suffix
         ];
         let requireRestart = false;
-        if (this.state._readOnly != this.state.readOnly) {
+        if (this.state._readOnly !== this.state.readOnly) {
             if (this.state.readOnly) {
                 cmd.push("--enable-readonly");
             } else {
                 cmd.push("--disable-readonly");
             }
         }
-        if (this.state._requireIndex != this.state.requireIndex) {
+        if (this.state._requireIndex !== this.state.requireIndex) {
             if (this.state.requireIndex) {
                 cmd.push("--require-index");
             } else {
                 cmd.push("--ignore-index");
             }
         }
-        if (this.state._cachememsize != this.state.cachememsize) {
+        if (this.state._cachememsize !== this.state.cachememsize) {
             cmd.push("--cache-memsize=" + this.state.cachememsize);
             requireRestart = true;
         }
-        if (this.state._cachesize != this.state.cachesize) {
+        if (this.state._cachesize !== this.state.cachesize) {
             cmd.push("--cache-size=" + this.state.cachesize);
             requireRestart = true;
         }
-        if (this.state._dncachememsize != this.state.dncachememsize) {
+        if (this.state._dncachememsize !== this.state.dncachememsize) {
             cmd.push("--dncache-memsize=" + this.state.dncachememsize);
             requireRestart = true;
         }
-        if (this.state._dbstate != this.state.dbstate) {
+        if (this.state._dbstate !== this.state.dbstate) {
             cmd.push("--state=" + this.state.dbstate);
             requireRestart = true;
         }
@@ -818,7 +821,7 @@ export class Suffix extends React.Component {
                         if (requireRestart) {
                             this.props.addNotification(
                                 "warning",
-                                msg + "You must restart the Directory Server for these changes to take effect."
+                                msg + _("You must restart the Directory Server for these changes to take effect.")
                             );
                         }
                         this.setState({
@@ -834,7 +837,7 @@ export class Suffix extends React.Component {
                         }
                         this.props.addNotification(
                             "error",
-                            `Error updating suffix configuration - ${msg}`
+                            cockpit.format(_("Error updating suffix configuration - $0"), msg)
                         );
                         this.setState({
                             savingConfig: false
@@ -847,31 +850,31 @@ export class Suffix extends React.Component {
     // Render the component
     //
     render () {
-        let suffixIcon = faTree;
-        if (this.props.dbtype == "subsuffix") {
-            suffixIcon = faLeaf;
+        let SuffixIcon = FolderIcon;
+        if (this.props.dbtype === "subsuffix") {
+            SuffixIcon = LeafIcon;
         }
         const { dropdownIsOpen, activeTabKey } = this.state;
 
         const dropdownItems = [
-            <DropdownItem key="import" component="button" onClick={this.showImportModal} title="Import an LDIF file to initialize the database">
-                Initialize Suffix
+            <DropdownItem key="import" component="button" onClick={this.handleShowImportModal} title={_("Import an LDIF file to initialize the database")}>
+                {_("Initialize Suffix")}
             </DropdownItem>,
-            <DropdownItem key="export" component="button" onClick={this.showExportModal} title="Export the database to an LDIF file">
-                Export Suffix
+            <DropdownItem key="export" component="button" onClick={this.handleShowExportModal} title={_("Export the database to an LDIF file")}>
+                {_("Export Suffix")}
             </DropdownItem>,
-            <DropdownItem key="reindex" component="button" onClick={this.showReindexConfirm} title="Reindex the entire database">
-                Reindex Suffix
+            <DropdownItem key="reindex" component="button" onClick={this.handleShowReindexConfirm} title={_("Reindex the entire database")}>
+                {_("Reindex Suffix")}
             </DropdownItem>,
-            <DropdownItem key="subSuffix" component="button" onClick={this.showSubSuffixModal} title="Create a sub-suffix under this suffix">
-                Create Sub-Suffix
+            <DropdownItem key="subSuffix" component="button" onClick={this.handleShowSubSuffixModal} title={_("Create a sub-suffix under this suffix")}>
+                {_("Create Sub-Suffix")}
             </DropdownItem>,
-            <DropdownItem key="dbLink" component="button" onClick={this.showLinkModal} title="Create a database chaining link subtree">
-                Create Database Link
+            <DropdownItem key="dbLink" component="button" onClick={this.handleShowLinkModal} title={_("Create a database chaining link subtree")}>
+                {_("Create Database Link")}
             </DropdownItem>,
             <DropdownSeparator key="separator" />,
-            <DropdownItem key="deleteSuffix" component="button" onClick={this.showDeleteConfirm} title="This will permanently delete the database">
-                Delete Suffix
+            <DropdownItem key="deleteSuffix" component="button" onClick={this.handleShowDeleteConfirm} title={_("This will permanently delete the database")}>
+                {_("Delete Suffix")}
             </DropdownItem>,
         ];
 
@@ -879,22 +882,24 @@ export class Suffix extends React.Component {
             <div id="suffix-page">
                 <Grid>
                     <GridItem className="ds-suffix-header" span={9}>
-                        <FontAwesomeIcon size="sm" icon={suffixIcon} />&nbsp;&nbsp;{this.props.suffix} (<i>{this.props.bename}</i>)
-                        <FontAwesomeIcon
-                            className="ds-left-margin ds-refresh"
-                            icon={faSyncAlt}
-                            title="Refresh suffix"
+                        <SuffixIcon />
+                        &nbsp;&nbsp;{this.props.suffix} (<i>{this.props.bename}</i>)
+                        <Button 
+                            variant="plain"
+                            aria-label={_("Refresh suffix")}
                             onClick={() => this.props.reload(this.props.suffix)}
-                        />
+                        >
+                            <SyncAltIcon />
+                        </Button>
                     </GridItem>
                     <GridItem span={3}>
                         <Dropdown
                             className="ds-float-right"
                             position={DropdownPosition.right}
-                            onSelect={this.onSelect}
+                            onSelect={this.handleSelect}
                             toggle={
-                                <DropdownToggle id="suffix-dropdown" isPrimary onToggle={this.onToggle}>
-                                    Suffix Tasks
+                                <DropdownToggle id="suffix-dropdown" onToggle={(event, isOpen) => this.handleToggle(event, isOpen)}>
+                                    {_("Suffix Tasks")}
                                 </DropdownToggle>
                             }
                             isOpen={dropdownIsOpen}
@@ -904,8 +909,8 @@ export class Suffix extends React.Component {
                 </Grid>
 
                 <div className="ds-sub-header">
-                    <Tabs activeKey={activeTabKey} onSelect={this.handleNavSelect}>
-                        <Tab eventKey={0} title={<TabTitleText>Settings</TabTitleText>}>
+                    <Tabs isFilled activeKey={activeTabKey} onSelect={this.handleNavSelect}>
+                        <Tab eventKey={0} title={<TabTitleText>{_("Settings")}</TabTitleText>}>
                             <SuffixConfig
                                 cachememsize={this.state.cachememsize}
                                 cachesize={this.state.cachesize}
@@ -914,13 +919,13 @@ export class Suffix extends React.Component {
                                 readOnly={this.state.readOnly}
                                 requireIndex={this.state.requireIndex}
                                 autoTuning={this.state.autoTuning}
-                                handleChange={this.handleConfigChange}
-                                saveHandler={this.saveSuffixConfig}
+                                handleChange={this.onConfigChange}
+                                handleSave={this.saveSuffixConfig}
                                 saving={this.state.savingConfig}
                                 saveBtnDisabled={this.state.saveBtnDisabled}
                             />
                         </Tab>
-                        <Tab eventKey={1} title={<TabTitleText>Referrals</TabTitleText>}>
+                        <Tab eventKey={1} title={<TabTitleText>{_("Referrals")}</TabTitleText>}>
                             <SuffixReferrals
                                 rows={this.props.data.refRows}
                                 suffix={this.props.suffix}
@@ -930,7 +935,7 @@ export class Suffix extends React.Component {
                                 key={this.state.refRows}
                             />
                         </Tab>
-                        <Tab eventKey={2} title={<TabTitleText>Indexes</TabTitleText>}>
+                        <Tab eventKey={2} title={<TabTitleText>{_("Indexes")}</TabTitleText>}>
                             <SuffixIndexes
                                 systemIndexRows={this.props.data.systemIndexRows}
                                 indexRows={this.props.data.indexRows}
@@ -940,7 +945,7 @@ export class Suffix extends React.Component {
                                 reload={this.props.reloadIndexes}
                             />
                         </Tab>
-                        <Tab eventKey={3} title={<TabTitleText>VLV Indexes</TabTitleText>}>
+                        <Tab eventKey={3} title={<TabTitleText>{_("VLV Indexes")}</TabTitleText>}>
                             <VLVIndexes
                                 suffix={this.props.suffix}
                                 serverId={this.props.serverId}
@@ -951,7 +956,7 @@ export class Suffix extends React.Component {
                                 key={this.props.vlvTableKey}
                             />
                         </Tab>
-                        <Tab eventKey={4} title={<TabTitleText>Encrypted Attributes</TabTitleText>}>
+                        <Tab eventKey={4} title={<TabTitleText>{_("Encrypted Attributes")}</TabTitleText>}>
                             <AttrEncryption
                                 rows={this.props.data.encAttrsRows}
                                 suffix={this.props.suffix}
@@ -966,21 +971,21 @@ export class Suffix extends React.Component {
                 <DoubleConfirmModal
                     showModal={this.state.showDeleteConfirm}
                     closeHandler={this.closeDeleteConfirm}
-                    handleChange={this.handleChange}
+                    handleChange={this.onChange}
                     actionHandler={this.doDelete}
                     spinning={this.state.modalSpinning}
                     item={this.props.suffix}
                     checked={this.state.modalChecked}
-                    mTitle="Delete Suffix"
-                    mMsg="Are you really sure you want to delete this suffix?"
-                    mSpinningMsg="Deleting suffix ..."
-                    mBtnName="Delete Suffix"
+                    mTitle={_("Delete Suffix")}
+                    mMsg={_("Are you really sure you want to delete this suffix?")}
+                    mSpinningMsg={_("Deleting suffix ...")}
+                    mBtnName={_("Delete Suffix")}
                 />
                 <CreateLinkModal
                     showModal={this.state.showLinkModal}
                     closeHandler={this.closeLinkModal}
-                    handleChange={this.handleLinkChange}
-                    handleSelectChange={this.handleLinkOnSelect}
+                    handleChange={this.onLinkChange}
+                    handleSelectChange={this.onLinkOnSelect}
                     saveHandler={this.createLink}
                     suffix={this.props.suffix}
                     starttls_checked={this.state.createUseStartTLS}
@@ -992,8 +997,8 @@ export class Suffix extends React.Component {
                 <CreateSubSuffixModal
                     showModal={this.state.showSubSuffixModal}
                     closeHandler={this.closeSubSuffixModal}
-                    handleChange={this.handleSubSuffixChange}
-                    handleSelectChange={this.handleSubSuffixOnSelect}
+                    handleChange={this.onSubSuffixChange}
+                    handleSelectChange={this.onSubSuffixOnSelect}
                     saveHandler={this.createSubSuffix}
                     suffix={this.props.suffix}
                     error={this.state.errObj}
@@ -1004,48 +1009,48 @@ export class Suffix extends React.Component {
                 <ImportModal
                     showModal={this.state.showImportModal}
                     closeHandler={this.closeImportModal}
-                    handleChange={this.handleChange}
+                    handleChange={this.onChange}
                     saveHandler={this.doImport}
                     showConfirmImport={this.showConfirmLDIFImport}
                     rows={this.props.LDIFRows}
                     suffix={this.props.suffix}
-                    saveBtnDisabled={this.state.ldifLocation == ""}
+                    saveBtnDisabled={this.state.ldifLocation === ""}
                 />
                 <DoubleConfirmModal
                     showModal={this.state.showConfirmLDIFImport}
                     closeHandler={this.closeConfirmLDIFImport}
-                    handleChange={this.handleChange}
+                    handleChange={this.onChange}
                     actionHandler={this.importLDIF}
                     spinning={this.state.modalSpinning}
                     item={this.state.importLDIFName}
                     checked={this.state.modalChecked}
-                    mTitle="Initialize Database From LDIF"
-                    mMsg="Are you sure you want to initialize the database (it will permanently overwrite the current database)?"
-                    mSpinningMsg="Initializing Database ..."
-                    mBtnName="Initialize Database"
+                    mTitle={_("Initialize Database From LDIF")}
+                    mMsg={_("Are you sure you want to initialize the database (it will permanently overwrite the current database)?")}
+                    mSpinningMsg={_("Initializing Database ...")}
+                    mBtnName={_("Initialize Database")}
                 />
                 <ExportModal
                     showModal={this.state.showExportModal}
                     closeHandler={this.closeExportModal}
-                    handleChange={this.handleExportChange}
+                    handleChange={this.onExportChange}
                     saveHandler={this.doExport}
                     spinning={this.state.exportSpinner}
                     error={this.state.errObj}
                     includeReplData={this.state.includeReplData}
-                    saveBtnDisabled={this.state.ldifLocation == ""}
+                    saveBtnDisabled={this.state.ldifLocation === ""}
                 />
                 <DoubleConfirmModal
                     showModal={this.state.showReindexConfirm}
                     closeHandler={this.closeReindexConfirm}
-                    handleChange={this.handleChange}
+                    handleChange={this.onChange}
                     actionHandler={this.doReindex}
                     spinning={this.state.modalSpinning}
                     item={this.props.suffix}
                     checked={this.state.modalChecked}
-                    mTitle="Reindex All Attributes"
-                    mMsg="Are you sure you want to reindex all the attribute indexes?"
-                    mSpinningMsg="Reindexing Database ..."
-                    mBtnName="Reindex"
+                    mTitle={_("Reindex All Attributes")}
+                    mMsg={_("Are you sure you want to reindex all the attribute indexes?")}
+                    mSpinningMsg={_("Reindexing Database ...")}
+                    mBtnName={_("Reindex")}
                 />
             </div>
         );
