@@ -19,16 +19,11 @@ import {
     TextContent,
     TextVariants,
 } from "@patternfly/react-core";
-import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
-import {
-    faClone,
-    faLeaf,
-    faTree,
-    faSyncAlt
-} from '@fortawesome/free-solid-svg-icons';
-import '@fortawesome/fontawesome-svg-core/styles.css';
+import { SyncAltIcon, TreeIcon, CloneIcon, LeafIcon } from "@patternfly/react-icons";
 import PropTypes from "prop-types";
-import { log_cmd, valid_dn } from "../tools.jsx";
+import { log_cmd, valid_dn, callCmdStreamPassword } from "../tools.jsx";
+
+const _ = cockpit.gettext;
 
 export class ReplSuffix extends React.Component {
     constructor (props) {
@@ -48,6 +43,7 @@ export class ReplSuffix extends React.Component {
             enableBindPW: "",
             enableBindPWConfirm: "",
             enableBindGroupDN: "",
+            enableSpinning: false,
             disabled: true, // Disable repl enable button
             // Disable replication
             showDisableReplModal: false,
@@ -57,19 +53,19 @@ export class ReplSuffix extends React.Component {
             modalSpinning: false,
         };
 
-        this.onMinus = () => {
+        this.handleMinusChange = () => {
             this.setState({
                 enableRID: Number(this.state.enableRID) - 1
             });
         };
-        this.onNumberChange = (event) => {
+        this.handleNumberChange = (event) => {
             const newValue = isNaN(event.target.value) ? 0 : Number(event.target.value);
             this.setState({
                 enableRID: newValue > 65534 ? 65534 : newValue < 1 ? 1 : newValue
             });
         };
 
-        this.onPlus = () => {
+        this.handlePlusChange = () => {
             this.setState({
                 enableRID: Number(this.state.enableRID) + 1
             });
@@ -85,9 +81,9 @@ export class ReplSuffix extends React.Component {
 
         // General bindings
         this.handleReplChange = this.handleReplChange.bind(this);
-        this.handleEnableChange = this.handleEnableChange.bind(this);
+        this.onEnableChange = this.onEnableChange.bind(this);
         this.validateEnable = this.validateEnable.bind(this);
-        this.handleChange = this.handleChange.bind(this);
+        this.onChange = this.onChange.bind(this);
         this.handleNavSelect = this.handleNavSelect.bind(this);
         this.disableReplication = this.disableReplication.bind(this);
         this.enableReplication = this.enableReplication.bind(this);
@@ -114,22 +110,23 @@ export class ReplSuffix extends React.Component {
         } else {
             // Enable replication
             this.setState({
-                showEnableReplModal: true
+                showEnableReplModal: true,
+                enableSpinning: false,
             });
         }
     }
 
-    handleChange (e) {
+    onChange (e) {
         const value = e.target.type === 'checkbox' ? e.target.checked : e.target.value;
         let valueErr = false;
         const errObj = this.state.errObj;
-        if (value == "") {
+        if (value === "") {
             valueErr = true;
         }
         errObj[e.target.id] = valueErr;
         this.setState({
             [e.target.id]: value,
-            errObj: errObj
+            errObj
         });
     }
 
@@ -139,14 +136,14 @@ export class ReplSuffix extends React.Component {
 
         const dnAttrs = ['enableBindDN', 'enableBindGroupDN'];
         for (const attr of dnAttrs) {
-            if (this.state[attr] != "" && (!valid_dn(this.state[attr]) || !this.state[attr].includes(','))) {
+            if (this.state[attr] !== "" && (!valid_dn(this.state[attr]) || !this.state[attr].includes(','))) {
                 all_good = false;
                 errObj[attr] = true;
             }
         }
 
         if (this.state.enableBindDN) {
-            if (this.state.enableBindPW == "" || this.state.enableBindPW != this.state.enableBindPWConfirm) {
+            if (this.state.enableBindPW === "" || this.state.enableBindPW !== this.state.enableBindPWConfirm) {
                 errObj.enableBindPW = true;
                 errObj.enableBindPWConfirm = true;
                 all_good = false;
@@ -154,12 +151,12 @@ export class ReplSuffix extends React.Component {
         }
 
         this.setState({
-            errObj: errObj,
+            errObj,
             disabled: !all_good
         });
     }
 
-    handleEnableChange (e) {
+    onEnableChange (e) {
         const value = e.target.value;
         const attr = e.target.id;
 
@@ -176,31 +173,31 @@ export class ReplSuffix extends React.Component {
 
     enableReplication () {
         // First, Validate
-        if (this.state.enableBindDN != "" && !valid_dn(this.state.enableBindDN)) {
+        if (this.state.enableBindDN !== "" && !valid_dn(this.state.enableBindDN)) {
             this.props.addNotification(
                 "error",
-                `The Bind DN is not a valid DN (Distinguished Name) ${this.state.enableBindDN}`
+                cockpit.format(_("The Bind DN is not a valid DN (Distinguished Name) $0"), this.state.enableBindDN)
             );
             return;
         }
-        if (this.state.enableBindGroupDN != "" && !valid_dn(this.state.enableBindGroupDN)) {
+        if (this.state.enableBindGroupDN !== "" && !valid_dn(this.state.enableBindGroupDN)) {
             this.props.addNotification(
                 "error",
-                `The Group DN is not a valid DN (Distinguished Name)`
+                _("The Group DN is not a valid DN (Distinguished Name)")
             );
             return;
         }
-        if (this.state.enableBindPW != this.state.enableBindPWConfirm) {
+        if (this.state.enableBindPW !== this.state.enableBindPWConfirm) {
             this.props.addNotification(
                 "error",
-                `The Bind DN passwords do not match`
+                _("The Bind DN passwords do not match")
             );
             return;
         }
-        if (this.state.enableRID != "" && (this.state.enableRID < 1 || this.state.enableRID > 65534)) {
+        if (this.state.enableRID !== "" && (this.state.enableRID < 1 || this.state.enableRID > 65534)) {
             this.props.addNotification(
                 "error",
-                `The Replica ID is not in the valid range of 1 - 65534`
+                _("The Replica ID is not in the valid range of 1 - 65534")
             );
             return;
         }
@@ -211,38 +208,41 @@ export class ReplSuffix extends React.Component {
             'replication', 'enable', '--suffix=' + this.props.suffix,
             '--role=' + this.state.enableRole
         ];
-        if (this.state.enableBindDN != "") {
+        let passwd = "";
+        if (this.state.enableBindDN !== "") {
             cmd.push('--bind-dn=' + this.state.enableBindDN);
         }
-        if (this.state.enableBindPW != "") {
-            cmd.push('--bind-passwd=' + this.state.enableBindPW);
+        if (this.state.enableBindPW !== "") {
+            passwd = this.state.enableBindPW;
         }
-        if (this.state.enableBindGroupDN != "") {
+        if (this.state.enableBindGroupDN !== "") {
             cmd.push('--bind-group-dn=' + this.state.enableBindGroupDN);
         }
-        if (this.state.enableRole == "Supplier") {
+        if (this.state.enableRole === "Supplier") {
             cmd.push('--replica-id=' + this.state.enableRID);
         }
 
+        this.setState({
+            enableSpinning: true
+        });
+
         this.props.disableTree();
-        log_cmd('enableReplication', 'Enable replication', cmd);
-        cockpit
-                .spawn(cmd, { superuser: true, err: "message" })
-                .done(content => {
-                    this.props.reload(1);
-                    this.props.addNotification(
-                        "success",
-                        `Successfully enabled replication for "${this.props.suffix}"`
-                    );
-                })
-                .fail(err => {
-                    this.props.reload(1);
-                    const errMsg = JSON.parse(err);
-                    this.props.addNotification(
-                        "error",
-                        `Failed to enable replication for "${this.props.suffix}" - ${errMsg.desc}`
-                    );
-                });
+
+        // Something changed, perform the update
+        const config = {
+            cmd,
+            promptArg: "--bind-passwd-prompt",
+            passwd,
+            addNotification: this.props.addNotification,
+            success_msg: cockpit.format(_("Successfully enabled replication for $0"), this.props.suffix),
+            error_msg: cockpit.format(_("Failed to enable replication for $0"), this.props.suffix),
+            state_callback: () => {},
+            reload_func: this.props.reload,
+            reload_arg: "1",
+            funcName: "enableReplication",
+            funcDesc: "Enable replication"
+        };
+        callCmdStreamPassword(config);
     }
 
     closeDisableReplModal () {
@@ -253,23 +253,32 @@ export class ReplSuffix extends React.Component {
 
     disableReplication () {
         this.props.disableTree();
+        this.setState({
+            modalSpinning: true
+        });
         const cmd = ['dsconf', '-j', 'ldapi://%2fvar%2frun%2fslapd-' + this.props.serverId + '.socket', 'replication', 'disable', '--suffix=' + this.props.suffix];
         log_cmd('disableReplication', 'Disable replication', cmd);
         cockpit
                 .spawn(cmd, { superuser: true, err: "message" })
                 .done(content => {
                     this.props.reload(1);
+                    this.setState({
+                        modalSpinning: false
+                    });
                     this.props.addNotification(
                         "success",
-                        `Successfully disabled replication for "${this.props.suffix}"`
+                        cockpit.format(_("Successfully disabled replication for $0"), this.props.suffix)
                     );
                 })
                 .fail(err => {
                     this.props.reload(1);
+                    this.setState({
+                        modalSpinning: false
+                    });
                     const errMsg = JSON.parse(err);
                     this.props.addNotification(
                         "error",
-                        `Failed to disable replication for "${this.props.suffix}" - ${errMsg.desc}`
+                        cockpit.format(_("Failed to disable replication for $0 - $1"), this.props.suffix, errMsg.desc)
                     );
                 });
     }
@@ -280,29 +289,29 @@ export class ReplSuffix extends React.Component {
     render () {
         let spinning = "";
         let spintext = "";
-        let suffixIcon = faTree;
+        let SuffixIcon = TreeIcon;
         if (this.props.replicated) {
-            suffixIcon = faClone;
+            SuffixIcon = CloneIcon;
         } else {
-            if (this.props.repl == "subsuffix") {
-                suffixIcon = faLeaf;
+            if (this.props.repl === "subsuffix") {
+                SuffixIcon = LeafIcon;
             }
         }
         if (this.props.spinning) {
             spinning =
                 <Spinner className="ds-margin-top ds-margin-left ds-inline-spinner" size="sm" />;
             spintext =
-                <font size="2"><i>Refreshing</i></font>;
+                <font size="2"><i>{_("Refreshing")}</i></font>;
         }
         let suffixClass = "ds-margin-top-lg";
         if (this.props.disabled) {
             suffixClass = "ds-margin-top-lg ds-disabled";
         }
 
-        let enabledContent =
+        let enabledContent = (
             <div className={suffixClass}>
-                <Tabs activeKey={this.state.activeTabKey} onSelect={this.handleNavSelect}>
-                    <Tab eventKey={0} title={<TabTitleText>Configuration</TabTitleText>}>
+                <Tabs isFilled activeKey={this.state.activeTabKey} onSelect={this.handleNavSelect}>
+                    <Tab eventKey={0} title={<TabTitleText>{_("Configuration")}</TabTitleText>}>
                         <ReplConfig
                             suffix={this.props.suffix}
                             role={this.props.role}
@@ -313,7 +322,7 @@ export class ReplSuffix extends React.Component {
                             reloadConfig={this.props.reloadConfig}
                         />
                     </Tab>
-                    <Tab eventKey={1} title={<TabTitleText>Agreements <font size="2">({this.props.agmtRows.length})</font></TabTitleText>}>
+                    <Tab eventKey={1} title={<TabTitleText>{_("Agreements ")}<font size="2">({this.props.agmtRows.length})</font></TabTitleText>}>
                         <ReplAgmts
                             suffix={this.props.suffix}
                             serverId={this.props.serverId}
@@ -325,7 +334,7 @@ export class ReplSuffix extends React.Component {
                             key={this.props.agmtRows}
                         />
                     </Tab>
-                    <Tab eventKey={2} title={<TabTitleText>Winsync Agreements <font size="2">({this.props.winsyncRows.length})</font></TabTitleText>}>
+                    <Tab eventKey={2} title={<TabTitleText>{_("Winsync Agreements ")}<font size="2">({this.props.winsyncRows.length})</font></TabTitleText>}>
                         <WinsyncAgmts
                             suffix={this.props.suffix}
                             serverId={this.props.serverId}
@@ -337,7 +346,7 @@ export class ReplSuffix extends React.Component {
                             key={this.props.winsyncRows}
                         />
                     </Tab>
-                    <Tab eventKey={3} title={<TabTitleText>Change Log</TabTitleText>}>
+                    <Tab eventKey={3} title={<TabTitleText>{_("Change Log")}</TabTitleText>}>
                         <Changelog
                             suffix={this.props.suffix}
                             serverId={this.props.serverId}
@@ -349,7 +358,7 @@ export class ReplSuffix extends React.Component {
                             key={this.props.data}
                         />
                     </Tab>
-                    <Tab eventKey={4} title={<TabTitleText>RUV's & Tasks</TabTitleText>}>
+                    <Tab eventKey={4} title={<TabTitleText>{_("RUV's & Tasks")}</TabTitleText>}>
                         <ReplRUV
                             suffix={this.props.suffix}
                             serverId={this.props.serverId}
@@ -362,25 +371,27 @@ export class ReplSuffix extends React.Component {
                         />
                     </Tab>
                 </Tabs>
-            </div>;
+            </div>
+        );
 
         let replActionButton = "";
         if (this.props.replicated) {
-            replActionButton =
+            replActionButton = (
                 <Button
                     className="ds-float-right"
                     variant="danger"
                     onClick={this.handleReplChange}
-                    title="Disable replication, and remove all replication agreements."
+                    title={_("Disable replication, and remove all replication agreements.")}
                 >
-                    Disable
-                </Button>;
+                    {_("Disable")}
+                </Button>
+            );
         } else {
-            enabledContent =
+            enabledContent = (
                 <div className="ds-center ds-margin-top-xlg">
                     <TextContent>
                         <Text component={TextVariants.h3}>
-                            Replication is not enabled for this suffix
+                            {_("Replication is not enabled for this suffix")}
                         </Text>
                     </TextContent>
                     <Button
@@ -388,22 +399,25 @@ export class ReplSuffix extends React.Component {
                         onClick={this.handleReplChange}
                         className="ds-margin-top-lg"
                     >
-                        Enable Replication
+                        {_("Enable Replication")}
                     </Button>
-                </div>;
+                </div>
+            );
         }
 
         return (
             <div id="suffix-page">
                 <Grid>
                     <GridItem className="ds-suffix-header" span={8}>
-                        <FontAwesomeIcon size="sm" icon={suffixIcon} />&nbsp;&nbsp;{this.props.suffix}
-                        <FontAwesomeIcon
-                            className="ds-left-margin ds-refresh"
-                            icon={faSyncAlt}
-                            title="Refresh replication settings for this suffix"
+                        <SuffixIcon />
+                        &nbsp;&nbsp;{this.props.suffix}
+                        <Button 
+                            variant="plain"
+                            aria-label={_("Refresh replication settings for this suffix")}
                             onClick={() => this.props.reload(false)}
-                        />
+                        >
+                            <SyncAltIcon />
+                        </Button>
                         {spinning} {spintext}
                     </GridItem>
                     <GridItem span={4}>
@@ -414,30 +428,30 @@ export class ReplSuffix extends React.Component {
                 <EnableReplModal
                     showModal={this.state.showEnableReplModal}
                     closeHandler={this.closeEnableReplModal}
-                    handleChange={this.handleEnableChange}
+                    handleChange={this.onEnableChange}
                     saveHandler={this.enableReplication}
-                    spinning={this.state.addManagerSpinning}
+                    spinning={this.state.enableSpinning}
                     enableRole={this.state.enableRole}
                     enableRID={this.state.enableRID}
                     enableBindDN={this.state.enableBindDN}
                     disabled={this.state.disabled}
-                    onMinus={this.onMinus}
-                    onNumberChange={this.onNumberChange}
-                    onPlus={this.onPlus}
+                    onMinus={this.handleMinusChange}
+                    onNumberChange={this.handleNumberChange}
+                    onPlus={this.handlePlusChange}
                     error={this.state.errObj}
                 />
                 <DoubleConfirmModal
                     showModal={this.state.showDisableReplModal}
                     closeHandler={this.closeDisableReplModal}
-                    handleChange={this.handleChange}
+                    handleChange={this.onChange}
                     actionHandler={this.disableReplication}
                     spinning={this.state.modalSpinning}
                     item={this.props.suffix}
                     checked={this.state.modalChecked}
-                    mTitle="Disable Replication"
-                    mMsg="Are you sure you want to disable replication for this suffix?"
-                    mSpinningMsg="Disabling Replication ..."
-                    mBtnName="Disable Replication"
+                    mTitle={_("Disable Replication")}
+                    mMsg={_("Are you sure you want to disable replication for this suffix?")}
+                    mSpinningMsg={_("Disabling Replication ...")}
+                    mBtnName={_("Disable Replication")}
                 />
             </div>
         );
