@@ -30,12 +30,15 @@ import LinkedAttributes from "./lib/plugins/linkedAttributes.jsx";
 import ManagedEntries from "./lib/plugins/managedEntries.jsx";
 import MemberOf from "./lib/plugins/memberOf.jsx";
 import PassthroughAuthentication from "./lib/plugins/passthroughAuthentication.jsx";
+import PAMPassthroughAuthentication from "./lib/plugins/pamPassThru.jsx";
 import ReferentialIntegrity from "./lib/plugins/referentialIntegrity.jsx";
 import RetroChangelog from "./lib/plugins/retroChangelog.jsx";
 import RootDNAccessControl from "./lib/plugins/rootDNAccessControl.jsx";
 import USNPlugin from "./lib/plugins/usn.jsx";
 import WinSync from "./lib/plugins/winsync.jsx";
 
+const _ = cockpit.gettext;
+
 export class Plugins extends React.Component {
     componentDidUpdate(prevProps) {
         if (this.props.wasActiveList.includes(5)) {
@@ -88,9 +91,9 @@ export class Plugins extends React.Component {
             currentPluginPrecedence: ""
         };
 
-        this.handleSelect = result => {
+        this.handleSelect = (_event, item) => {
             this.setState({
-                activePlugin: result.itemId
+                activePlugin: item.itemId
             });
         };
 
@@ -149,7 +152,7 @@ export class Plugins extends React.Component {
                             })
                             .fail(err => {
                                 const errMsg = JSON.parse(err);
-                                this.props.addNotification("error", `Failed to get objectClasses - ${errMsg.desc}`);
+                                this.props.addNotification("error", cockpit.format(_("Failed to get objectClasses - $0"), errMsg.desc));
                             });
                 });
     }
@@ -184,7 +187,7 @@ export class Plugins extends React.Component {
         cockpit
                 .spawn(cmd, { superuser: true, err: "message" })
                 .done(content => {
-                    var myObject = JSON.parse(content);
+                    const myObject = JSON.parse(content);
                     const pluginTableKey = this.state.pluginTableKey + 1;
                     if (this.state.firstLoad) {
                         this.setState(prevState => ({
@@ -194,12 +197,12 @@ export class Plugins extends React.Component {
                             },
                             rows: myObject.items,
                             firstLoad: false,
-                            pluginTableKey: pluginTableKey,
+                            pluginTableKey,
                         }));
                     } else {
                         this.setState({
                             rows: myObject.items,
-                            pluginTableKey: pluginTableKey,
+                            pluginTableKey,
                         });
                     }
                 })
@@ -207,7 +210,7 @@ export class Plugins extends React.Component {
                     const errMsg = JSON.parse(err);
                     this.props.addNotification(
                         "error",
-                        `${errMsg.desc} error during plugin loading`
+                        cockpit.format(_("$0 error during plugin loading"), errMsg.desc)
                     );
                 });
     }
@@ -258,7 +261,7 @@ export class Plugins extends React.Component {
                     basicPluginSuccess = true;
                     this.props.addNotification(
                         "success",
-                        `Plugin ${data.name} was successfully modified`
+                        cockpit.format(_("Plugin $0 was successfully modified"), data.name)
                     );
                     this.pluginList();
                     this.closePluginModal();
@@ -271,7 +274,7 @@ export class Plugins extends React.Component {
                     } else {
                         this.props.addNotification(
                             "error",
-                            `${errMsg.desc} error during ${data.name} modification`
+                            cockpit.format(_("$0 error during $1 modification"), errMsg.desc, data.name)
                         );
                     }
                     this.closePluginModal();
@@ -295,7 +298,7 @@ export class Plugins extends React.Component {
                                     if (!basicPluginSuccess) {
                                         this.props.addNotification(
                                             "success",
-                                            `Plugin ${data.name} was successfully modified`
+                                            cockpit.format(_("Plugin $0 was successfully modified"), data.name)
                                         );
                                     }
                                     this.pluginList();
@@ -311,13 +314,13 @@ export class Plugins extends React.Component {
                                         if (basicPluginSuccess) {
                                             this.props.addNotification(
                                                 "success",
-                                                `Plugin ${data.name} was successfully modified`
+                                                cockpit.format(_("Plugin $0 was successfully modified"), data.name)
                                             );
                                             this.pluginList();
                                         }
                                         this.props.addNotification(
                                             "error",
-                                            `${errMsg.desc} error during ${data.name} modification`
+                                            cockpit.format(_("$0 error during $1 modification"), errMsg.desc, data.name)
                                         );
                                     }
                                     this.toggleLoading();
@@ -367,8 +370,7 @@ export class Plugins extends React.Component {
                     this.pluginList();
                     this.props.addNotification(
                         "warning",
-                        `${this.state.togglePluginName} plugin was successfully set to "${new_status}".
-                        Please, restart the instance.`
+                        cockpit.format(_("$0 plugin was successfully set to $1. Please, restart the instance."), this.state.togglePluginName, new_status)
                     );
                     this.setState({
                         modalSpinning: false,
@@ -379,7 +381,7 @@ export class Plugins extends React.Component {
                     const errMsg = JSON.parse(err);
                     this.props.addNotification(
                         "error",
-                        `Error during ${this.state.togglePluginName} plugin modification - ${errMsg.desc}`
+                        cockpit.format(_("Error during $0 plugin modification - $1"), this.state.togglePluginName, errMsg.desc)
                     );
                     // toggleLoadingHandler();
                     this.setState({
@@ -393,9 +395,9 @@ export class Plugins extends React.Component {
         const pluginRow = this.state.rows.find(row => row.cn[0] === plugin_name);
         if (pluginRow) {
             if (pluginRow['nsslapd-pluginEnabled'][0] === "on") {
-                return <div className="ds-ok-icon"><CheckCircleIcon title="Plugin is enabled" className="ds-icon-sm" />{name}</div>;
+                return <div className="ds-ok-icon"><CheckCircleIcon title={_("Plugin is enabled")} className="ds-icon-sm" />{name}</div>;
             } else {
-                return <div className="ds-disabled-icon"><BanIcon title="Plugin is disabled" className="ds-icon-sm" />{name}</div>;
+                return <div className="ds-disabled-icon"><BanIcon title={_("Plugin is disabled")} className="ds-icon-sm" />{name}</div>;
             }
         } else {
             // Might be attribute uniqueness that just has individual entries
@@ -403,12 +405,12 @@ export class Plugins extends React.Component {
             if (otherRows.length > 0) {
                 for (const plugin of otherRows) {
                     if (plugin['nsslapd-pluginEnabled'][0] === "on") {
-                        return <div className="ds-ok-icon"><CheckCircleIcon title="Plugin is enabled" className="ds-icon-sm" />{name}</div>;
+                        return <div className="ds-ok-icon"><CheckCircleIcon title={_("Plugin is enabled")} className="ds-icon-sm" />{name}</div>;
                     }
                 }
-                return <div className="ds-disabled-icon"><BanIcon title="Plugin is disabled" className="ds-icon-sm" />{name}</div>;
+                return <div className="ds-disabled-icon"><BanIcon title={_("Plugin is disabled")} className="ds-icon-sm" />{name}</div>;
             } else {
-                return <div className="ds-disabled-icon"><ResourcesEmptyIcon title="Plugin is not configured" className="ds-icon-sm" />{name}</div>;
+                return <div className="ds-disabled-icon"><ResourcesEmptyIcon title={_("Plugin is not configured")} className="ds-icon-sm" />{name}</div>;
             }
         }
     }
@@ -417,7 +419,7 @@ export class Plugins extends React.Component {
         const selectPlugins = {
             allPlugins: {
                 name: "All Plugins",
-                icon: <div><ListIcon className="ds-icon-sm" />All Plugins</div>,
+                icon: <div><ListIcon className="ds-icon-sm" />{_("All Plugins")}</div>,
                 component: (
                     <PluginTable
                         key={this.state.pluginTableKey}
@@ -545,8 +547,8 @@ export class Plugins extends React.Component {
                 )
             },
             passthroughAuthentication: {
-                name: "Pass Through Auth",
-                icon: this.getIconAndName("Pass Through Auth", "Pass Through Authentication"),
+                name: "LDAP Pass Through Auth",
+                icon: this.getIconAndName("LDAP Pass Through Auth", "Pass Through Authentication"),
                 component: (
                     <PassthroughAuthentication
                         rows={this.state.rows}
@@ -561,6 +563,23 @@ export class Plugins extends React.Component {
                     />
                 )
             },
+            pamPassthroughAuthentication: {
+                name: "PAM Pass Through Auth",
+                icon: this.getIconAndName("PAM Pass Through Auth", "PAM Pass Through Auth"),
+                component: (
+                    <PAMPassthroughAuthentication
+                        rows={this.state.rows}
+                        serverId={this.props.serverId}
+                        savePluginHandler={this.savePlugin}
+                        pluginListHandler={this.pluginList}
+                        addNotification={this.props.addNotification}
+                        toggleLoadingHandler={this.toggleLoading}
+                        wasActiveList={this.props.wasActiveList}
+                        attributes={this.state.attributes}
+                        key={this.props.wasActiveList}
+                    />
+                )
+            },
             winsync: {
                 name: "Posix Winsync",
                 icon: this.getIconAndName("Posix Winsync", "Posix Winsync API"),
@@ -650,25 +669,23 @@ export class Plugins extends React.Component {
                 <div className="ds-margin-top-xlg ds-center" hidden={!this.state.firstLoad}>
                     <TextContent>
                         <Text component={TextVariants.h3}>
-                            Loading Plugins ...
+                            {_("Loading Plugins ...")}
                         </Text>
                     </TextContent>
                     <Spinner className="ds-margin-top-lg" size="xl" />
                 </div>
                 <div hidden={this.state.firstLoad} className={this.state.loading ? "ds-disabled" : ""}>
                     <Grid className="ds-margin-top-xlg" hasGutter>
-                        <GridItem span={3}>
-                            <div>
-                                <Nav key={this.state.pluginTableKey} theme="light" onSelect={this.handleSelect}>
-                                    <NavList>
-                                        {Object.entries(selectPlugins).map(([id, item]) => (
-                                            <NavItem key={item.name} itemId={item.name} isActive={this.state.activePlugin === item.name}>
-                                                {item.icon}
-                                            </NavItem>
-                                        ))}
-                                    </NavList>
-                                </Nav>
-                            </div>
+                        <GridItem span={3} className="ds-vert-scroll">
+                            <Nav key={this.state.pluginTableKey} theme="light" onSelect={(event, item) => this.handleSelect(event, item)}>
+                                <NavList>
+                                    {Object.entries(selectPlugins).map(([id, item]) => (
+                                        <NavItem key={item.name} itemId={item.name} isActive={this.state.activePlugin === item.name}>
+                                            {item.icon}
+                                        </NavItem>
+                                    ))}
+                                </NavList>
+                            </Nav>
                         </GridItem>
                         <GridItem className="ds-indent-md" span={9}>
                             {Object.entries(selectPlugins).filter(plugin => plugin[1].name === this.state.activePlugin)
@@ -687,14 +704,16 @@ export class Plugins extends React.Component {
                         spinning={this.state.modalSpinning}
                         item={this.state.togglePluginName}
                         checked={this.state.modalChecked}
-                        mTitle={this.state.togglePluginEnabled ? "Disable Plugin" : "Enable Plugin"}
+                        mTitle={this.state.togglePluginEnabled ? _("Disable Plugin") : _("Enable Plugin")}
                         mMsg={this.state.togglePluginEnabled
-                            ? "Are you really sure you want to disable this plugin?  Disabling some plugins can cause the server to not start, please use caution."
-                            : "Are you sure you want to enable this plugin?"}
+                            ? _("Are you really sure you want to disable this plugin?  Disabling some plugins can cause the server to not start, please use caution.")
+                            : _("Are you sure you want to enable this plugin?")}
                         mSpinningMsg={this.state.togglePluginEnabled
-                            ? "Disabling plugin ..." : "Enabling plugin ..."}
+                            ? _("Disabling plugin ...")
+                            : _("Enabling plugin ...")}
                         mBtnName={this.state.togglePluginEnabled
-                            ? "Disable Plugin" : "Enable Plugin"}
+                            ? _("Disable Plugin")
+                            : _("Enable Plugin")}
                     />
                 </div>
             </div>
