@@ -10,7 +10,7 @@
 from lib389.schema import Schema, Resolver
 from lib389.backend import Backends
 from lib389.migrate.openldap.config import olOverlayType
-from lib389.plugins import MemberOfPlugin, ReferentialIntegrityPlugin, AttributeUniquenessPlugins
+from lib389.plugins import MemberOfPlugin, ReferentialIntegrityPlugin, AttributeUniquenessPlugins, PassThroughAuthenticationPlugin, PAMPassThroughAuthPlugin, PAMPassThroughAuthConfigs
 import ldap
 import os
 from ldif import LDIFParser
@@ -98,8 +98,11 @@ class DatabaseReindex(MigrationAction):
         log.info(f" * Database Reindex -> {self.suffix}")
 
 class ImportTransformer(LDIFParser):
-    def __init__(self, f_import, f_outport, exclude_attributes_set):
+    def __init__(self, f_import, f_outport, exclude_attributes_set, exclude_objectclass_set):
         self.exclude_attributes_set = exclude_attributes_set
+        # Already all lowered by the db ldif import
+
+        self.exclude_objectclass_set = exclude_objectclass_set
         self.f_outport = f_outport
         self.writer = LDIFWriter(self.f_outport)
         super().__init__(f_import)
@@ -123,6 +126,11 @@ class ImportTransformer(LDIFParser):
             # Not found
             pass
 
+        # We remove the objectclass set and reinsert minus anything excluded.
+        e_oc_set = entry[oc_a]
+        e_oc_set = [ oc for oc in e_oc_set if oc.lower() not in self.exclude_objectclass_set ]
+        entry[oc_a] = e_oc_set
+
         # Strip anything in the exclude set.
         for attr in self.exclude_attributes_set:
             try:
@@ -132,14 +140,30 @@ class ImportTransformer(LDIFParser):
                 # Not found, move on.
                 pass
 
+        # If userPassword is present AND it is a SASL map, then we migrate it to the nsSaslauthId
+        # type to prevent account password confusion.
+        try:
+            pw_a = amap['userpassword']
+            pw = entry[pw_a][0]
+            if pw.startswith(b'{SASL}'):
+                entry.pop(pw_a)
+                sasl_id = pw.replace(b'{SASL}', b'')
+                # Add the marker objectClass and the map attr
+                entry[oc_a] += [b'nsSaslauthAccount']
+                entry['nsSaslauthId'] = [sasl_id]
+        except:
+            # Not found, move on.
+            pass
+
         # Write it out
         self.writer.unparse(dn, entry)
 
 class DatabaseLdifImport(MigrationAction):
-    def __init__(self, suffix, ldif_path, exclude_attributes_set):
+    def __init__(self, suffix, ldif_path, exclude_attributes_set, exclude_objectclass_set):
         self.suffix = suffix
         self.ldif_path = ldif_path
         self.exclude_attributes_set = exclude_attributes_set
+        self.exclude_objectclass_set = [ x.lower() for x in exclude_objectclass_set ]
 
     def apply(self, inst):
         # Create a unique op id.
@@ -148,7 +172,7 @@ class DatabaseLdifImport(MigrationAction):
 
         with open(self.ldif_path, 'r') as f_import:
             with open(op_path, 'w') as f_outport:
-                p = ImportTransformer(f_import, f_outport, self.exclude_attributes_set)
+                p = ImportTransformer(f_import, f_outport, self.exclude_attributes_set, self.exclude_objectclass_set)
                 p.parse()
 
         be = Backends(inst).get(self.suffix)
@@ -159,10 +183,10 @@ class DatabaseLdifImport(MigrationAction):
         task.wait()
 
     def __unicode__(self):
-        return f"DatabaseLdifImport -> {self.suffix} {self.ldif_path}"
+        return f"DatabaseLdifImport -> {self.suffix} {self.ldif_path}, {self.exclude_attributes_set}, {self.exclude_objectclass_set}"
 
     def display_plan(self, log):
-        log.info(f" * Database Import Ldif -> {self.suffix} from {self.ldif_path} - excluding entry attributes = [{self.exclude_attributes_set}]")
+        log.info(f" * Database Import Ldif -> {self.suffix} from {self.ldif_path} - excluding entry attributes = [{self.exclude_attributes_set}], excluding entry objectclasses = [{self.exclude_objectclass_set}]")
 
     def display_post(self, log):
         log.info(f" * [ ] - Review Database Imported Content is Correct -> {self.suffix}")
@@ -211,20 +235,21 @@ class SchemaAttributeInconsistent(MigrationAction):
         log.info(f" * Schema Skip Inconsistent Attribute -> {self.attr.names[0]} ({self.attr.oid})")
 
     def display_post(self, log):
-        log.info(f" * [ ] - Review Schema Inconsistent Attribute -> {self.obj.names[0]} ({self.obj.oid})")
+        log.info(f" * [ ] - Review Schema Inconsistent Attribute -> {self.attr.names[0]} ({self.attr.oid})")
 
 class SchemaAttributeAmbiguous(MigrationAction):
-    def __init__(self, attr):
+    def __init__(self, attr, overlaps):
         self.attr = attr
+        self.overlaps = overlaps
 
     def __unicode__(self):
         return f"SchemaAttributeAmbiguous -> {self.attr.__unicode__()}"
 
     def display_plan(self, log):
-        log.info(f" * Schema Skip Abmiguous Attribute -> {self.attr.names[0]} ({self.attr.oid})")
+        log.info(f" * Schema Skip Abmiguous Attribute -> {self.attr.names[0]} ({self.attr.oid}) overlaps {self.overlaps}")
 
     def display_post(self, log):
-        log.info(f" * [ ] - Review Schema Ambiguous Attribute -> {self.obj.names[0]} ({self.obj.oid})")
+        log.info(f" * [ ] - Review Schema Ambiguous Attribute -> {self.attr.names[0]} ({self.attr.oid}) overlaps {self.overlaps}")
 
 class SchemaAttributeInvalid(MigrationAction):
     def __init__(self, attr, reason):
@@ -433,6 +458,86 @@ class PluginUniqueConfigure(MigrationAction):
     def display_plan(self, log):
         log.info(f" * Plugin:Unique Add Attribute and Suffix -> {self.attr} {self.suffix}")
 
+
+class PluginPwdPolicyAudit(MigrationAction):
+    def __init__(self, suffix, inst_name):
+        self.suffix = suffix
+        self.inst_name = inst_name
+
+    def __unicode__(self):
+        return f"PluginPwdPolicyAudit -> {self.suffix}"
+
+    def display_post(self, log):
+        log.info(f" * [ ] - Review Imported Password Policies for accounts under {self.suffix}.")
+        log.info(f" * [ ] - Enable Password Policy for new accounts in {self.suffix}. See `dsconf {self.inst_name} localpwp --help` ")
+
+
+class PluginPassThroughDisable(MigrationAction):
+    def __init__(self):
+        pass
+
+    def apply(self, inst):
+        pta = PassThroughAuthenticationPlugin(inst)
+        pta.disable()
+
+    def __unicode__(self):
+        return "PluginPassThroughDisable"
+
+    def display_plan(self, log):
+        log.info(f" * Plugin:PassThrough Disable")
+
+    def display_post(self, log):
+        pass
+
+
+class PluginPAMPassThroughEnable(MigrationAction):
+    def __init__(self):
+        pass
+
+    def apply(self, inst):
+        pta = PAMPassThroughAuthPlugin(inst)
+        pta.enable()
+
+    def __unicode__(self):
+        return "PAMPassThroughAuthPlugin"
+
+    def display_plan(self, log):
+        log.info(f" * Plugin:PamPassThrough Enable")
+
+    def display_post(self, log):
+        pass
+
+
+class PluginPAMPassThroughConfigure(MigrationAction):
+    def __init__(self, suffix):
+        self.suffix = suffix
+
+    def apply(self, inst):
+        ptas = PAMPassThroughAuthConfigs(inst)
+        # PAMPassThroughAuthConfig
+        pta = ptas.ensure_state(properties = {
+            'cn': 'saslauthd',
+            'pamService': 'saslauthd',
+            'pamIncludeSuffix': self.suffix,
+            'pamFilter': '(objectClass=nsSaslauthAccount)',
+            'pamIdAttr': 'nsSaslauthId',
+            'pamIDMapMethod': 'ENTRY',
+            'pamSecure': 'TRUE',
+            'pamFallback': 'FALSE',
+            'pamModuleIsThreadSafe': 'TRUE',
+            'pamMissingSuffix': 'ALLOW',
+        })
+
+    def __unicode__(self):
+        return "PluginPAMPassThroughConfigure"
+
+    def display_plan(self, log):
+        log.info(f" * Plugin:PamPassThrough Configure")
+
+    def display_post(self, log):
+        log.info(f" * [ ] - Review Plugin:PAMPassThrough Migrated SASLauthd Configuration is Correct")
+
+
 class PluginUnknownManual(MigrationAction):
     def __init__(self, overlay):
         self.overlay = overlay
@@ -504,6 +609,29 @@ class Migration(object):
             '1.3.6.1.1.1.2.17', # automount
             # This schema is buggy, we always skip it as we know the 389 version is correct.
             '0.9.2342.19200300.100.4.14',
+            # ppolicy overlay - these names conflict to 389-ds values, but have the same function.
+            '1.3.6.1.4.1.42.2.27.8.1.2', # pwdMinAge
+            '1.3.6.1.4.1.42.2.27.8.1.3', # pwdMaxAge
+            '1.3.6.1.4.1.42.2.27.8.1.4', # pwdInHistory
+            '1.3.6.1.4.1.42.2.27.8.1.6', # pwdMinLength
+            '1.3.6.1.4.1.42.2.27.8.1.7', # pwdExpireWarning
+            '1.3.6.1.4.1.42.2.27.8.1.8', # pwdGraceAuthNLimit
+            '1.3.6.1.4.1.42.2.27.8.1.9', # pwdLockout
+            '1.3.6.1.4.1.42.2.27.8.1.10', # pwdLockoutDuration
+            '1.3.6.1.4.1.42.2.27.8.1.11', # pwdMaxFailure
+            '1.3.6.1.4.1.42.2.27.8.1.12', # pwdFailureCountInt
+            '1.3.6.1.4.1.42.2.27.8.1.13', # pwdMustChange
+            '1.3.6.1.4.1.42.2.27.8.1.14', # pwdAllowUserChange
+            '1.3.6.1.4.1.4754.2.99.1', # pwdPolicyChecker
+            '1.3.6.1.4.1.42.2.27.8.2.1', # pwdPolicy objectClass
+            # Openldap supplies some schema which conflicts to ours, skip them
+            'NetscapeLDAPattributeType:198', # memberUrl
+            'NetscapeLDAPobjectClass:33', # groupOfURLs
+
+            # Dynamic Directory Services can't be supported due to missing syntax oid below, so we
+            # exclude the "otherwise" supported attrs / ocs
+            'DynGroupAttr:1', # dgIdentity
+            'DynGroupOC:1', # dgIdentityAux
         ] + skip_schema_oids)
         self._schema_oid_unsupported = set([
             # RFC4517 othermailbox syntax is not supported on 389.
@@ -519,12 +647,44 @@ class Migration(object):
             # Pilot DSA needs dsaquality
             '0.9.2342.19200300.100.4.21',
             '0.9.2342.19200300.100.4.22',
-
+            # These ppolicy module are unsupported so they are skipped.
+            '1.3.6.1.4.1.42.2.27.8.1.1', # pwdAttribute
+            '1.3.6.1.4.1.42.2.27.8.1.5', # pwdCheckQuality
+            '1.3.6.1.4.1.42.2.27.8.1.15', # pwdSafeModify
+            '1.3.6.1.4.1.4754.1.99.1', # pwdCheckModule
+            '1.3.6.1.4.1.42.2.27.8.1.30', # pwdMaxRecordedFailure
+            # OpenLDAP dynamic directory services defines an internal
+            # oid ( 1.3.6.1.4.1.4203.666.2.7 )for dynamic group authz, but has very little docs about this.
+            'DynGroupAttr:2', # dgAuthz
         ])
+
         self._skip_entry_attributes = set(
-            ['entrycsn', 'structuralobjectclass'] +
+            [
+                # Core openldap attrs that we can't use, and don't matter.
+                'entrycsn',
+                'contextcsn',
+                'structuralobjectclass',
+                # pwd attributes from ppolicy which are not supported.
+                'pwdattribute',
+                'pwdcheckquality',
+                'pwdsafemodify',
+                'pwdcheckmodule',
+                'pwdmaxrecordedfailure',
+                # ppolicy attr that isn't in schema that isn't supported.
+                'pwdfailuretime',
+                # dds attributes we don't support
+                'dgidentity',
+                'dgauthz'
+            ] +
             [x.lower() for x in skip_entry_attributes]
         )
+        # These tend to be be from overlays we don't support
+        self._skip_entry_objectclasses = set([
+            'pwdpolicychecker',
+            'pwdpolicy',
+            'dgidentityaux'
+        ])
+
         self._gen_migration_plan()
 
     def __unicode__(self):
@@ -579,7 +739,7 @@ class Migration(object):
                 self.plan.append(SchemaClassUnsupported(obj))
                 continue
             if obj.oid.lower().startswith('x-'):
-                self.plan.append(SchemaClassInvalid(attr, "descr-oid format is ambiguous with x- prefix. You MUST use a numeric oid."))
+                self.plan.append(SchemaClassInvalid(obj, "descr-oid format is ambiguous with x- prefix. You MUST use a numeric oid."))
                 continue
             # For the attr, find if anything has a name overlap in any capacity.
             overlaps = [ ds_obj for ds_obj in schema_objects if ds_obj.oid == obj.oid]
@@ -641,8 +801,16 @@ class Migration(object):
             elif overlay.otype == olOverlayType.UNIQUE:
                 for attr in overlay.attrs:
                     self.plan.append(PluginUniqueConfigure(oldb.suffix, attr, oldb.uuid))
+            elif overlay.otype == olOverlayType.PPOLICY:
+                self.plan.append(PluginPwdPolicyAudit(oldb.suffix, self.inst.serverid))
             else:
                 raise Exception("Unknown overlay type, this is a bug!")
+        # We can't programatically detect the use of sasl authd. We can however
+        # create a configuration that "isn't harmful" in the case it's not used,
+        # and that does work in the case that it is present.
+        self.plan.append(PluginPassThroughDisable())
+        self.plan.append(PluginPAMPassThroughEnable())
+        self.plan.append(PluginPAMPassThroughConfigure(oldb.suffix))
 
 
     def _gen_db_plan(self):
@@ -670,7 +838,7 @@ class Migration(object):
         if self.ldifs is None:
             return
         for (suffix, ldif_path) in self.ldifs.items():
-            self.plan.append(DatabaseLdifImport(suffix, ldif_path, self._skip_entry_attributes))
+            self.plan.append(DatabaseLdifImport(suffix, ldif_path, self._skip_entry_attributes, self._skip_entry_objectclasses))
 
     def _gen_migration_plan(self):
         """Order of this module is VERY important!!!
@@ -698,7 +866,12 @@ class Migration(object):
             count += 1
 
         # Before we do post actions, restart the instance.
-        self.inst.restart()
+        if not self.inst._containerised:
+            self.inst.restart()
+        else:
+            log.info("Restart your instance now!")
+            _ = input("Press enter when complete")
+            self.inst.open()
 
         # Then do post actions
         count = 1
