@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2021 Red Hat, Inc.
+# Copyright (C) 2023 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -88,18 +88,18 @@ class Task(DSLdapObject):
                 return None
         return None
 
-    def wait(self, timeout=120):
+    def wait(self, timeout=120, sleep_interval=2):
         """Wait until task is complete."""
 
-        count = 0
-        if timeout is None:
+        time_passed = 0
+        if timeout is None or timeout == 0:
             self._log.debug("No timeout is set, this may take a long time ...")
 
-        while timeout is None or count < timeout:
+        while timeout is None or timeout == 0 or time_passed < timeout:
             if self.is_complete():
                 break
-            count = count + 1
-            time.sleep(2)
+            time_passed = time_passed + sleep_interval
+            time.sleep(sleep_interval)
 
     def create(self, rdn=None, properties={}, basedn=None):
         """Create a Task entry
@@ -168,7 +168,7 @@ class FixupLinkedAttributesTask(Task):
 
 
 class MemberUidFixupTask(Task):
-    """A single instance of memberOf task entry
+    """A single instance of posix group fix task entry
 
     :param instance: An instance
     :type instance: lib389.DirSrv
@@ -525,7 +525,7 @@ class Tasks(object):
             entry.setValues('nsIncludeSuffix', suffix)
 
         # start the task and possibly wait for task completion
-        self.conn.add_s(entry)
+        self.conn.add_s(entry, escapehatch='i am sure')
 
         exitCode = 0
         warningCode = 0
@@ -598,7 +598,7 @@ class Tasks(object):
             entry.setValues('nsExportReplica', 'true')
 
         # start the task and possibly wait for task completion
-        self.conn.add_s(entry)
+        self.conn.add_s(entry, escapehatch='i am sure')
         exitCode = 0
         warningCode = 0
         if args and args.get(TASK_WAIT, False):
@@ -649,7 +649,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add the backup task (%s)", dn)
             return -1
@@ -706,7 +706,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add the backup task (%s)", dn)
             return -1
@@ -834,7 +834,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add the index task for %s", attrname)
             return -1
@@ -915,7 +915,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add the memberOf fixup task")
             return -1
@@ -977,7 +977,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add the fixup tombstone task")
             return -1
@@ -1006,12 +1006,13 @@ class Tasks(object):
         return exitCode
 
     def automemberRebuild(self, suffix=DEFAULT_SUFFIX, scope='sub',
-                          filterstr='objectclass=top', args=None):
+                          filterstr='objectclass=top', cleanup=False, args=None):
         '''
-        @param suffix - The suffix the task should examine - defualt is
+        @param suffix - The suffix the task should examine - default is
                         "dc=example,dc=com"
         @param scope - The scope of the search to find entries
-        @param fitlerstr - THe search filter to find entries
+        @param fitlerstr - The search filter to find entries
+        @param cleanup - reset/clear the old group mmeberships prior to rebuilding
         @param args - is a dictionary that contains modifier of the task
                 wait: True/[False] - If True,  waits for the completion of
                                      the task before to return
@@ -1027,10 +1028,12 @@ class Tasks(object):
         entry.setValues('basedn', suffix)
         entry.setValues('filter', filterstr)
         entry.setValues('scope', scope)
+        if cleanup:
+            entry.setValues('cleanup', 'yes')
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Automember Rebuild Membership task")
             return -1
@@ -1086,7 +1089,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Automember Export Updates task")
             return -1
@@ -1137,7 +1140,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Automember Map Updates task")
             return -1
@@ -1182,7 +1185,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Fixup Linked Attributes task")
             return -1
@@ -1226,7 +1229,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Schema Reload task")
             return -1
@@ -1271,7 +1274,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add fixupWinsyncMembers 'memberuid task'")
             return -1
@@ -1318,7 +1321,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Syntax Validate task")
             return -1
@@ -1369,7 +1372,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add USN tombstone cleanup task")
             return -1
@@ -1420,7 +1423,7 @@ class Tasks(object):
             entry.setValues('logchanges', logchanges)
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Sysconfig Reload task")
             return -1
@@ -1472,7 +1475,7 @@ class Tasks(object):
             entry.setValues('replica-force-cleaning', 'yes')
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add cleanAllRUV task")
             return (dn, -1)
@@ -1527,7 +1530,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add Abort cleanAllRUV task")
             return (dn, -1)
@@ -1581,7 +1584,7 @@ class Tasks(object):
 
         # start the task and possibly wait for task completion
         try:
-            self.conn.add_s(entry)
+            self.conn.add_s(entry, escapehatch='i am sure')
         except ldap.ALREADY_EXISTS:
             self.log.error("Fail to add upgradedb task")
             return -1
