@@ -1,8 +1,6 @@
-
 #include <inttypes.h>
 
 int32_t
-do_nothing_really_well_abcdef() {
+do_nothing_really_well_abcdef(void) {
     return 0;
 }
-
